@@ -1,4 +1,4 @@
-/* $Id: smallmap_type.hpp 21645 2010-12-26 17:47:00Z alberth $ */
+/* $Id: smallmap_type.hpp 24741 2012-11-14 22:50:30Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,8 +17,8 @@
 
 /**
  * Simple pair of data. Both types have to be POD ("Plain Old Data")!
- * @tvar T Key type.
- * @tvar U Value type.
+ * @tparam T Key type.
+ * @tparam U Value type.
  */
 template <typename T, typename U>
 struct SmallPair {
@@ -26,15 +26,15 @@ struct SmallPair {
 	U second;
 
 	/** Initializes this Pair with data */
-	FORCEINLINE SmallPair(const T &first, const U &second) : first(first), second(second) { }
+	inline SmallPair(const T &first, const U &second) : first(first), second(second) { }
 };
 
 /**
  * Implementation of simple mapping class. Both types have to be POD ("Plain Old Data")!
  * It has inherited accessors from SmallVector().
- * @tvar T Key type.
- * @tvar U Value type.
- * @tvar S Unit of allocation.
+ * @tparam T Key type.
+ * @tparam U Value type.
+ * @tparam S Unit of allocation.
  *
  * @see SmallVector
  */
@@ -45,16 +45,29 @@ struct SmallMap : SmallVector<SmallPair<T, U>, S> {
 	typedef const Pair *const_iterator;
 
 	/** Creates new SmallMap. Data are initialized in SmallVector constructor */
-	FORCEINLINE SmallMap() { }
+	inline SmallMap() { }
 	/** Data are freed in SmallVector destructor */
-	FORCEINLINE ~SmallMap() { }
+	inline ~SmallMap() { }
 
 	/**
 	 * Finds given key in this map
 	 * @param key key to find
 	 * @return &Pair(key, data) if found, this->End() if not
 	 */
-	FORCEINLINE Pair *Find(const T &key)
+	inline const Pair *Find(const T &key) const
+	{
+		for (uint i = 0; i < this->items; i++) {
+			if (key == this->data[i].first) return &this->data[i];
+		}
+		return this->End();
+	}
+
+	/**
+	 * Finds given key in this map
+	 * @param key key to find
+	 * @return &Pair(key, data) if found, this->End() if not
+	 */
+	inline Pair *Find(const T &key)
 	{
 		for (uint i = 0; i < this->items; i++) {
 			if (key == this->data[i].first) return &this->data[i];
@@ -67,7 +80,7 @@ struct SmallMap : SmallVector<SmallPair<T, U>, S> {
 	 * @param key key to test
 	 * @return true iff the item is present
 	 */
-	FORCEINLINE bool Contains(const T &key)
+	inline bool Contains(const T &key) const
 	{
 		return this->Find(key) != this->End();
 	}
@@ -77,7 +90,7 @@ struct SmallMap : SmallVector<SmallPair<T, U>, S> {
 	 * @param pair pair to remove
 	 * @note it has to be pointer to pair in this map. It is overwritten by the last item.
 	 */
-	FORCEINLINE void Erase(Pair *pair)
+	inline void Erase(Pair *pair)
 	{
 		assert(pair >= this->Begin() && pair < this->End());
 		*pair = this->data[--this->items];
@@ -89,7 +102,7 @@ struct SmallMap : SmallVector<SmallPair<T, U>, S> {
 	 * @return true iff the key was found
 	 * @note last item is moved to its place, so don't increase your iterator if true is returned!
 	 */
-	FORCEINLINE bool Erase(const T &key)
+	inline bool Erase(const T &key)
 	{
 		for (uint i = 0; i < this->items; i++) {
 			if (key == this->data[i].first) {
@@ -106,7 +119,7 @@ struct SmallMap : SmallVector<SmallPair<T, U>, S> {
 	 * @param data data
 	 * @return true iff the key wasn't already present
 	 */
-	FORCEINLINE bool Insert(const T &key, const U &data)
+	inline bool Insert(const T &key, const U &data)
 	{
 		if (this->Contains(key)) return false;
 		Pair *n = this->Append();
@@ -121,7 +134,7 @@ struct SmallMap : SmallVector<SmallPair<T, U>, S> {
 	 * @return data belonging to this key
 	 * @note if this key wasn't present, new entry is created
 	 */
-	FORCEINLINE U &operator[](const T &key)
+	inline U &operator[](const T &key)
 	{
 		for (uint i = 0; i < this->items; i++) {
 			if (key == this->data[i].first) return this->data[i].second;
@@ -131,7 +144,7 @@ struct SmallMap : SmallVector<SmallPair<T, U>, S> {
 		return n->second;
 	}
 
-	FORCEINLINE void SortByKey()
+	inline void SortByKey()
 	{
 		QSortT(this->Begin(), this->items, KeySorter);
 	}
