@@ -1,4 +1,4 @@
-/* $Id: crashlog.cpp 20494 2010-08-14 18:01:55Z rubidium $ */
+/* $Id: crashlog.cpp 26538 2014-04-28 21:06:51Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -25,13 +25,19 @@
 #include "screenshot.h"
 #include "gfx_func.h"
 #include "network/network.h"
+#include "language.h"
+#include "fontcache.h"
 
 #include "ai/ai_info.hpp"
+#include "game/game.hpp"
+#include "game/game_info.hpp"
 #include "company_base.h"
 #include "company_func.h"
 
 #include <time.h>
 
+#include "safeguards.h"
+
 /* static */ const char *CrashLog::message = NULL;
 /* static */ char *CrashLog::gamelog_buffer = NULL;
 /* static */ const char *CrashLog::gamelog_last = NULL;
@@ -72,6 +78,12 @@ char *CrashLog::LogCompiler(char *buffer, const char *last) const
 	return buffer;
 }
 
+/**
+ * Writes OpenTTD's version to the buffer.
+ * @param buffer The begin where to write at.
+ * @param last   The last position in the buffer to write to.
+ * @return the position of the \c '\0' character after the buffer.
+ */
 char *CrashLog::LogOpenTTDVersion(char *buffer, const char *last) const
 {
 	return buffer + seprintf(buffer, last,
@@ -104,6 +116,13 @@ char *CrashLog::LogOpenTTDVersion(char *buffer, const char *last) const
 	);
 }
 
+/**
+ * Writes the (important) configuration settings to the buffer.
+ * E.g. graphics set, sound set, blitter and AIs.
+ * @param buffer The begin where to write at.
+ * @param last   The last position in the buffer to write to.
+ * @return the position of the \c '\0' character after the buffer.
+ */
 char *CrashLog::LogConfiguration(char *buffer, const char *last) const
 {
 	buffer += seprintf(buffer, last,
@@ -117,18 +136,30 @@ char *CrashLog::LogConfiguration(char *buffer, const char *last) const
 			" Sound driver: %s\n"
 			" Sound set:    %s (%u)\n"
 			" Video driver: %s\n\n",
-			BlitterFactoryBase::GetCurrentBlitter() == NULL ? "none" : BlitterFactoryBase::GetCurrentBlitter()->GetName(),
+			BlitterFactory::GetCurrentBlitter() == NULL ? "none" : BlitterFactory::GetCurrentBlitter()->GetName(),
 			BaseGraphics::GetUsedSet() == NULL ? "none" : BaseGraphics::GetUsedSet()->name,
 			BaseGraphics::GetUsedSet() == NULL ? UINT32_MAX : BaseGraphics::GetUsedSet()->version,
-			StrEmpty(_dynlang.curr_file) ? "none" : _dynlang.curr_file,
-			_music_driver == NULL ? "none" : _music_driver->GetName(),
+			_current_language == NULL ? "none" : _current_language->file,
+			MusicDriver::GetInstance() == NULL ? "none" : MusicDriver::GetInstance()->GetName(),
 			BaseMusic::GetUsedSet() == NULL ? "none" : BaseMusic::GetUsedSet()->name,
 			BaseMusic::GetUsedSet() == NULL ? UINT32_MAX : BaseMusic::GetUsedSet()->version,
 			_networking ? (_network_server ? "server" : "client") : "no",
-			_sound_driver == NULL ? "none" : _sound_driver->GetName(),
+			SoundDriver::GetInstance() == NULL ? "none" : SoundDriver::GetInstance()->GetName(),
 			BaseSounds::GetUsedSet() == NULL ? "none" : BaseSounds::GetUsedSet()->name,
 			BaseSounds::GetUsedSet() == NULL ? UINT32_MAX : BaseSounds::GetUsedSet()->version,
-			_video_driver == NULL ? "none" : _video_driver->GetName()
+			VideoDriver::GetInstance() == NULL ? "none" : VideoDriver::GetInstance()->GetName()
+	);
+
+	buffer += seprintf(buffer, last,
+			"Fonts:\n"
+			" Small:  %s\n"
+			" Medium: %s\n"
+			" Large:  %s\n"
+			" Mono:   %s\n\n",
+			FontCache::Get(FS_SMALL)->GetFontName(),
+			FontCache::Get(FS_NORMAL)->GetFontName(),
+			FontCache::Get(FS_LARGE)->GetFontName(),
+			FontCache::Get(FS_MONO)->GetFontName()
 	);
 
 	buffer += seprintf(buffer, last, "AI Configuration (local: %i):\n", (int)_local_company);
@@ -137,11 +168,13 @@ char *CrashLog::LogConfiguration(char *buffer, const char *last) const
 		if (c->ai_info == NULL) {
 			buffer += seprintf(buffer, last, " %2i: Human\n", (int)c->index);
 		} else {
-#ifdef ENABLE_AI
 			buffer += seprintf(buffer, last, " %2i: %s (v%d)\n", (int)c->index, c->ai_info->GetName(), c->ai_info->GetVersion());
-#endif /* ENABLE_AI */
 		}
 	}
+
+	if (Game::GetInfo() != NULL) {
+		buffer += seprintf(buffer, last, " GS: %s (v%d)\n", Game::GetInfo()->GetName(), Game::GetInfo()->GetVersion());
+	}
 	buffer += seprintf(buffer, last, "\n");
 
 	return buffer;
@@ -166,6 +199,9 @@ char *CrashLog::LogConfiguration(char *buffer, const char *last) const
 #ifdef WITH_ICU
 #	include <unicode/uversion.h>
 #endif /* WITH_ICU */
+#ifdef WITH_LZMA
+#	include <lzma.h>
+#endif
 #ifdef WITH_LZO
 #include <lzo/lzo1x.h>
 #endif
@@ -177,6 +213,12 @@ char *CrashLog::LogConfiguration(char *buffer, const char *last) const
 # include <zlib.h>
 #endif
 
+/**
+ * Writes information (versions) of the used libraries.
+ * @param buffer The begin where to write at.
+ * @param last   The last position in the buffer to write to.
+ * @return the position of the \c '\0' character after the buffer.
+ */
 char *CrashLog::LogLibraries(char *buffer, const char *last) const
 {
 	buffer += seprintf(buffer, last, "Libraries:\n");
@@ -208,6 +250,10 @@ char *CrashLog::LogLibraries(char *buffer, const char *last) const
 	buffer += seprintf(buffer, last, " ICU:        %s\n", buf);
 #endif /* WITH_ICU */
 
+#ifdef WITH_LZMA
+	buffer += seprintf(buffer, last, " LZMA:       %s\n", lzma_version_string());
+#endif
+
 #ifdef WITH_LZO
 	buffer += seprintf(buffer, last, " LZO:        %s\n", lzo_version_string());
 #endif
@@ -235,11 +281,21 @@ char *CrashLog::LogLibraries(char *buffer, const char *last) const
 	return buffer;
 }
 
+/**
+ * Helper function for printing the gamelog.
+ * @param s the string to print.
+ */
 /* static */ void CrashLog::GamelogFillCrashLog(const char *s)
 {
 	CrashLog::gamelog_buffer += seprintf(CrashLog::gamelog_buffer, CrashLog::gamelog_last, "%s\n", s);
 }
 
+/**
+ * Writes the gamelog data to the buffer.
+ * @param buffer The begin where to write at.
+ * @param last   The last position in the buffer to write to.
+ * @return the position of the \c '\0' character after the buffer.
+ */
 char *CrashLog::LogGamelog(char *buffer, const char *last) const
 {
 	CrashLog::gamelog_buffer = buffer;
@@ -248,6 +304,12 @@ char *CrashLog::LogGamelog(char *buffer, const char *last) const
 	return CrashLog::gamelog_buffer + seprintf(CrashLog::gamelog_buffer, last, "\n");
 }
 
+/**
+ * Fill the crash log buffer with all data of a crash log.
+ * @param buffer The begin where to write at.
+ * @param last   The last position in the buffer to write to.
+ * @return the position of the \c '\0' character after the buffer.
+ */
 char *CrashLog::FillCrashLog(char *buffer, const char *last) const
 {
 	time_t cur_time = time(NULL);
@@ -273,6 +335,15 @@ char *CrashLog::FillCrashLog(char *buffer, const char *last) const
 	return buffer;
 }
 
+/**
+ * Write the crash log to a file.
+ * @note On success the filename will be filled with the full path of the
+ *       crash log file. Make sure filename is at least \c MAX_PATH big.
+ * @param buffer The begin of the buffer to write to the disk.
+ * @param filename      Output for the filename of the written file.
+ * @param filename_last The last position in the filename buffer.
+ * @return true when the crash log was successfully written.
+ */
 bool CrashLog::WriteCrashLog(const char *buffer, char *filename, const char *filename_last) const
 {
 	seprintf(filename, filename_last, "%scrash.log", _personal_dir);
@@ -293,6 +364,14 @@ bool CrashLog::WriteCrashLog(const char *buffer, char *filename, const char *fil
 	return 0;
 }
 
+/**
+ * Write the (crash) savegame to a file.
+ * @note On success the filename will be filled with the full path of the
+ *       crash save file. Make sure filename is at least \c MAX_PATH big.
+ * @param filename      Output for the filename of the written file.
+ * @param filename_last The last position in the filename buffer.
+ * @return true when the crash save was successfully made.
+ */
 bool CrashLog::WriteSavegame(char *filename, const char *filename_last) const
 {
 	/* If the map array doesn't exist, saving will fail too. If the map got
@@ -311,16 +390,30 @@ bool CrashLog::WriteSavegame(char *filename, const char *filename_last) const
 	}
 }
 
+/**
+ * Write the (crash) screenshot to a file.
+ * @note On success the filename will be filled with the full path of the
+ *       screenshot. Make sure filename is at least \c MAX_PATH big.
+ * @param filename      Output for the filename of the written file.
+ * @param filename_last The last position in the filename buffer.
+ * @return true when the crash screenshot was successfully made.
+ */
 bool CrashLog::WriteScreenshot(char *filename, const char *filename_last) const
 {
 	/* Don't draw when we have invalid screen size */
 	if (_screen.width < 1 || _screen.height < 1 || _screen.dst_ptr == NULL) return false;
 
-	bool res = MakeScreenshot(SC_RAW, "crash");
+	bool res = MakeScreenshot(SC_CRASHLOG, "crash");
 	if (res) strecpy(filename, _full_screenshot_name, filename_last);
 	return res;
 }
 
+/**
+ * Makes the crash log, writes it to a file and then subsequently tries
+ * to make a crash dump and crash savegame. It uses DEBUG to write
+ * information like paths to the console.
+ * @return true when everything is made successfully.
+ */
 bool CrashLog::MakeCrashLog() const
 {
 	/* Don't keep looping logging crashes. */
@@ -376,14 +469,22 @@ bool CrashLog::MakeCrashLog() const
 	return ret;
 }
 
+/**
+ * Sets a message for the error message handler.
+ * @param message The error message of the error.
+ */
 /* static */ void CrashLog::SetErrorMessage(const char *message)
 {
 	CrashLog::message = message;
 }
 
+/**
+ * Try to close the sound/video stuff so it doesn't keep lingering around
+ * incorrect video states or so, e.g. keeping dpmi disabled.
+ */
 /* static */ void CrashLog::AfterCrashLogCleanup()
 {
-	if (_music_driver != NULL) _music_driver->Stop();
-	if (_sound_driver != NULL) _sound_driver->Stop();
-	if (_video_driver != NULL) _video_driver->Stop();
+	if (MusicDriver::GetInstance() != NULL) MusicDriver::GetInstance()->Stop();
+	if (SoundDriver::GetInstance() != NULL) SoundDriver::GetInstance()->Stop();
+	if (VideoDriver::GetInstance() != NULL) VideoDriver::GetInstance()->Stop();
 }
