@@ -1,4 +1,4 @@
-/* $Id: economy.cpp 26972 2014-10-06 20:10:07Z frosch $ */
+/* $Id: economy.cpp 27348 2015-07-30 18:45:29Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -54,6 +54,8 @@
 #include "table/strings.h"
 #include "table/pricebase.h"
 
+#include "safeguards.h"
+
 
 /* Initialize the cargo payment-pool */
 CargoPaymentPool _cargo_payment_pool("CargoPayment");
@@ -298,7 +300,7 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 		/* Single player cheated to AI company.
 		 * There are no spectators in single player, so we must pick some other company. */
 		assert(!_networking);
-		Backup<CompanyByte> cur_company(_current_company, FILE_LINE);
+		Backup<CompanyByte> cur_company2(_current_company, FILE_LINE);
 		Company *c;
 		FOR_ALL_COMPANIES(c) {
 			if (c->index != old_owner) {
@@ -306,7 +308,7 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 				break;
 			}
 		}
-		cur_company.Restore();
+		cur_company2.Restore();
 		assert(old_owner != _local_company);
 	}
 
@@ -429,11 +431,38 @@ void ChangeOwnershipOfCompanyItems(Owner old_owner, Owner new_owner)
 			FreeUnitIDGenerator(VEH_SHIP,  new_owner), FreeUnitIDGenerator(VEH_AIRCRAFT, new_owner)
 		};
 
+		/* Override company settings to new company defaults in case we need to convert them.
+		 * This is required as the CmdChangeServiceInt doesn't copy the supplied value when it is non-custom
+		 */
+		if (new_owner != INVALID_OWNER) {
+			Company *old_company = Company::Get(old_owner);
+			Company *new_company = Company::Get(new_owner);
+
+			old_company->settings.vehicle.servint_aircraft = new_company->settings.vehicle.servint_aircraft;
+			old_company->settings.vehicle.servint_trains = new_company->settings.vehicle.servint_trains;
+			old_company->settings.vehicle.servint_roadveh = new_company->settings.vehicle.servint_roadveh;
+			old_company->settings.vehicle.servint_ships = new_company->settings.vehicle.servint_ships;
+			old_company->settings.vehicle.servint_ispercent = new_company->settings.vehicle.servint_ispercent;
+		}
+
 		Vehicle *v;
 		FOR_ALL_VEHICLES(v) {
 			if (v->owner == old_owner && IsCompanyBuildableVehicleType(v->type)) {
 				assert(new_owner != INVALID_OWNER);
 
+				/* Correct default values of interval settings while maintaining custom set ones.
+				 * This prevents invalid values on mismatching company defaults being accepted.
+				 */
+				if (!v->ServiceIntervalIsCustom()) {
+					Company *new_company = Company::Get(new_owner);
+
+					/* Technically, passing the interval is not needed as the command will query the default value itself.
+					 * However, do not rely on that behaviour.
+					 */
+					int interval = CompanyServiceInterval(new_company, v->type);
+					DoCommand(v->tile, v->index, interval | (new_company->settings.vehicle.servint_ispercent << 17), DC_EXEC | DC_BANKRUPT, CMD_CHANGE_SERVICE_INT);
+				}
+
 				v->owner = new_owner;
 
 				/* Owner changes, clear cache */
@@ -1291,53 +1320,6 @@ static uint GetLoadAmount(Vehicle *v)
 	return load_amount;
 }
 
-/**
- * Reserves cargo if the full load order and improved_load is set or if the
- * current order allows autorefit.
- * @param st Station where the consist is loading at the moment.
- * @param u Front of the loading vehicle consist.
- * @param consist_capleft If given, save free capacities after reserving there.
- * @param next_station Station(s) the vehicle will stop at next.
- */
-static void ReserveConsist(Station *st, Vehicle *u, CargoArray *consist_capleft, StationIDStack next_station)
-{
-	Vehicle *next_cargo = u;
-	uint32 seen_cargos = 0;
-
-	while (next_cargo != NULL) {
-		if (next_cargo->cargo_cap == 0) {
-			/* No need to reserve for vehicles without capacity. */
-			next_cargo = next_cargo->Next();
-			continue;
-		}
-
-		CargoID current_cargo = next_cargo->cargo_type;
-
-		Vehicle *v = next_cargo;
-		SetBit(seen_cargos, current_cargo);
-		next_cargo = NULL;
-		for (; v != NULL; v = v->Next()) {
-			if (v->cargo_type != current_cargo) {
-				/* Save start point for next cargo type. */
-				if (next_cargo == NULL && !HasBit(seen_cargos, v->cargo_type)) next_cargo = v;
-				continue;
-			}
-
-			assert(v->cargo_cap >= v->cargo.RemainingCount());
-			uint cap = v->cargo_cap - v->cargo.RemainingCount();
-
-			/* Nothing to do if the vehicle is full */
-			if (cap > 0) {
-				cap -= st->goods[v->cargo_type].cargo.Reserve(cap, &v->cargo, st->xy, next_station);
-			}
-
-			if (consist_capleft != NULL) {
-				(*consist_capleft)[current_cargo] += cap;
-			}
-		}
-	}
-}
-
 /**
  * Iterate the articulated parts of a vehicle, also considering the special cases of "normal"
  * aircraft and double headed trains. Apply an action to each vehicle and immediately return false
@@ -1443,15 +1425,17 @@ struct FinalizeRefitAction
 	CargoArray &consist_capleft;  ///< Capacities left in the consist.
 	Station *st;                  ///< Station to reserve cargo from.
 	StationIDStack &next_station; ///< Next hops to reserve cargo for.
+	bool do_reserve;              ///< If the vehicle should reserve.
 
 	/**
 	 * Create a finalizing action.
 	 * @param consist_capleft Capacities left in the consist.
 	 * @param st Station to reserve cargo from.
 	 * @param next_station Next hops to reserve cargo for.
+	 * @param do_reserve If we should reserve cargo or just add up the capacities.
 	 */
-	FinalizeRefitAction(CargoArray &consist_capleft, Station *st, StationIDStack &next_station) :
-		consist_capleft(consist_capleft), st(st), next_station(next_station) {}
+	FinalizeRefitAction(CargoArray &consist_capleft, Station *st, StationIDStack &next_station, bool do_reserve) :
+		consist_capleft(consist_capleft), st(st), next_station(next_station), do_reserve(do_reserve) {}
 
 	/**
 	 * Reserve cargo from the station and update the remaining consist capacities with the
@@ -1461,8 +1445,10 @@ struct FinalizeRefitAction
 	 */
 	bool operator()(Vehicle *v)
 	{
-		this->st->goods[v->cargo_type].cargo.Reserve(v->cargo_cap - v->cargo.RemainingCount(),
-				&v->cargo, st->xy, next_station);
+		if (this->do_reserve) {
+			this->st->goods[v->cargo_type].cargo.Reserve(v->cargo_cap - v->cargo.RemainingCount(),
+					&v->cargo, st->xy, this->next_station);
+		}
 		this->consist_capleft[v->cargo_type] += v->cargo_cap - v->cargo.RemainingCount();
 		return true;
 	}
@@ -1488,7 +1474,8 @@ static void HandleStationRefit(Vehicle *v, CargoArray &consist_capleft, Station
 	/* Remove old capacity from consist capacity and collect refit mask. */
 	IterateVehicleParts(v_start, PrepareRefitAction(consist_capleft, refit_mask));
 
-	if (new_cid == CT_AUTO_REFIT) {
+	bool is_auto_refit = new_cid == CT_AUTO_REFIT;
+	if (is_auto_refit) {
 		/* Get a refittable cargo type with waiting cargo for next_station or INVALID_STATION. */
 		CargoID cid;
 		new_cid = v_start->cargo_type;
@@ -1524,11 +1511,83 @@ static void HandleStationRefit(Vehicle *v, CargoArray &consist_capleft, Station
 	}
 
 	/* Add new capacity to consist capacity and reserve cargo */
-	IterateVehicleParts(v_start, FinalizeRefitAction(consist_capleft, st, next_station));
+	IterateVehicleParts(v_start, FinalizeRefitAction(consist_capleft, st, next_station,
+			is_auto_refit || (v->First()->current_order.GetLoadType() & OLFB_FULL_LOAD) != 0));
 
 	cur_company.Restore();
 }
 
+struct ReserveCargoAction {
+	Station *st;
+	StationIDStack *next_station;
+
+	ReserveCargoAction(Station *st, StationIDStack *next_station) :
+		st(st), next_station(next_station) {}
+
+	bool operator()(Vehicle *v)
+	{
+		if (v->cargo_cap > v->cargo.RemainingCount()) {
+			st->goods[v->cargo_type].cargo.Reserve(v->cargo_cap - v->cargo.RemainingCount(),
+					&v->cargo, st->xy, *next_station);
+		}
+
+		return true;
+	}
+
+};
+
+/**
+ * Reserves cargo if the full load order and improved_load is set or if the
+ * current order allows autorefit.
+ * @param st Station where the consist is loading at the moment.
+ * @param u Front of the loading vehicle consist.
+ * @param consist_capleft If given, save free capacities after reserving there.
+ * @param next_station Station(s) the vehicle will stop at next.
+ */
+static void ReserveConsist(Station *st, Vehicle *u, CargoArray *consist_capleft, StationIDStack *next_station)
+{
+	/* If there is a cargo payment not all vehicles of the consist have tried to do the refit.
+	 * In that case, only reserve if it's a fixed refit and the equivalent of "articulated chain"
+	 * a vehicle belongs to already has the right cargo. */
+	bool must_reserve = !u->current_order.IsRefit() || u->cargo_payment == NULL;
+	for (Vehicle *v = u; v != NULL; v = v->Next()) {
+		assert(v->cargo_cap >= v->cargo.RemainingCount());
+
+		/* Exclude various ways in which the vehicle might not be the head of an equivalent of
+		 * "articulated chain". Also don't do the reservation if the vehicle is going to refit
+		 * to a different cargo and hasn't tried to do so, yet. */
+		if (!v->IsArticulatedPart() &&
+				(v->type != VEH_TRAIN || !Train::From(v)->IsRearDualheaded()) &&
+				(v->type != VEH_AIRCRAFT || Aircraft::From(v)->IsNormalAircraft()) &&
+				(must_reserve || u->current_order.GetRefitCargo() == v->cargo_type)) {
+			IterateVehicleParts(v, ReserveCargoAction(st, next_station));
+		}
+		if (consist_capleft == NULL || v->cargo_cap == 0) continue;
+		(*consist_capleft)[v->cargo_type] += v->cargo_cap - v->cargo.RemainingCount();
+	}
+}
+
+/**
+ * Update the vehicle's load_unload_ticks, the time it will wait until it tries to load or unload
+ * again. Adjust for overhang of trains and set it at least to 1.
+ * @param front The vehicle to be updated.
+ * @param st The station the vehicle is loading at.
+ * @param ticks The time it would normally wait, based on cargo loaded and unloaded.
+ */
+static void UpdateLoadUnloadTicks(Vehicle *front, const Station *st, int ticks)
+{
+	if (front->type == VEH_TRAIN) {
+		/* Each platform tile is worth 2 rail vehicles. */
+		int overhang = front->GetGroundVehicleCache()->cached_total_length - st->GetPlatformLength(front->tile) * TILE_SIZE;
+		if (overhang > 0) {
+			ticks <<= 1;
+			ticks += (overhang * ticks) / 8;
+		}
+	}
+	/* Always wait at least 1, otherwise we'll wait 'infinitively' long. */
+	front->load_unload_ticks = max(1, ticks);
+}
+
 /**
  * Loads/unload the vehicle if possible.
  * @param front the vehicle to be (un)loaded
@@ -1543,11 +1602,11 @@ static void LoadUnloadVehicle(Vehicle *front)
 	StationIDStack next_station = front->GetNextStoppingStation();
 	bool use_autorefit = front->current_order.IsRefit() && front->current_order.GetRefitCargo() == CT_AUTO_REFIT;
 	CargoArray consist_capleft;
-	if (_settings_game.order.improved_load &&
-			((front->current_order.GetLoadType() & OLFB_FULL_LOAD) != 0 || use_autorefit)) {
+	if (_settings_game.order.improved_load && use_autorefit ?
+			front->cargo_payment == NULL : (front->current_order.GetLoadType() & OLFB_FULL_LOAD) != 0) {
 		ReserveConsist(st, front,
 				(use_autorefit && front->load_unload_ticks != 0) ? &consist_capleft : NULL,
-				next_station);
+				&next_station);
 	}
 
 	/* We have not waited enough time till the next round of loading/unloading */
@@ -1561,7 +1620,7 @@ static void LoadUnloadVehicle(Vehicle *front)
 		return;
 	}
 
-	int unloading_time = 0;
+	int new_load_unload_ticks = 0;
 	bool dirty_vehicle = false;
 	bool dirty_station = false;
 
@@ -1636,7 +1695,7 @@ static void LoadUnloadVehicle(Vehicle *front)
 			if (amount_unloaded > 0) {
 				dirty_vehicle = true;
 				anything_unloaded = true;
-				unloading_time += amount_unloaded;
+				new_load_unload_ticks += amount_unloaded;
 
 				/* Deliver goods to the station */
 				st->time_since_unload = 0;
@@ -1730,7 +1789,7 @@ static void LoadUnloadVehicle(Vehicle *front)
 					AirportAnimationTrigger(st, AAT_STATION_CARGO_TAKEN, v->cargo_type);
 				}
 
-				unloading_time += loaded;
+				new_load_unload_ticks += loaded;
 
 				dirty_vehicle = dirty_station = true;
 			}
@@ -1762,15 +1821,18 @@ static void LoadUnloadVehicle(Vehicle *front)
 			 * on the vehicle type - the values here are those found in TTDPatch */
 			const uint gradual_loading_wait_time[] = { 40, 20, 10, 20 };
 
-			unloading_time = gradual_loading_wait_time[front->type];
+			new_load_unload_ticks = gradual_loading_wait_time[front->type];
 		}
 		/* We loaded less cargo than possible for all cargo types and it's not full
 		 * load and we're not supposed to wait any longer: stop loading. */
 		if (!anything_unloaded && full_load_amount == 0 && reservation_left == 0 && !(front->current_order.GetLoadType() & OLFB_FULL_LOAD) &&
-				front->current_order_time >= (uint)max(front->current_order.wait_time - front->lateness_counter, 0)) {
+				front->current_order_time >= (uint)max(front->current_order.GetTimetabledWait() - front->lateness_counter, 0)) {
 			SetBit(front->vehicle_flags, VF_STOP_LOADING);
 		}
+
+		UpdateLoadUnloadTicks(front, st, new_load_unload_ticks);
 	} else {
+		UpdateLoadUnloadTicks(front, st, 20); // We need the ticks for link refreshing.
 		bool finished_loading = true;
 		if (front->current_order.GetLoadType() & OLFB_FULL_LOAD) {
 			if (front->current_order.GetLoadType() == OLF_FULL_LOAD_ANY) {
@@ -1789,22 +1851,12 @@ static void LoadUnloadVehicle(Vehicle *front)
 			 * along them. Otherwise the vehicle could wait for cargo
 			 * indefinitely if it hasn't visited the other links yet, or if the
 			 * links die while it's loading. */
-			if (!finished_loading) LinkRefresher::Run(front);
+			if (!finished_loading) LinkRefresher::Run(front, true, true);
 		}
-		unloading_time = 20;
 
 		SB(front->vehicle_flags, VF_LOADING_FINISHED, 1, finished_loading);
 	}
 
-	if (front->type == VEH_TRAIN) {
-		/* Each platform tile is worth 2 rail vehicles. */
-		int overhang = front->GetGroundVehicleCache()->cached_total_length - st->GetPlatformLength(front->tile) * TILE_SIZE;
-		if (overhang > 0) {
-			unloading_time <<= 1;
-			unloading_time += (overhang * unloading_time) / 8;
-		}
-	}
-
 	/* Calculate the loading indicator fill percent and display
 	 * In the Game Menu do not display indicators
 	 * If _settings_client.gui.loading_indicators == 2, show indicators (bool can be promoted to int as 0 or 1 - results in 2 > 0,1 )
@@ -1821,9 +1873,6 @@ static void LoadUnloadVehicle(Vehicle *front)
 		}
 	}
 
-	/* Always wait at least 1, otherwise we'll wait 'infinitively' long. */
-	front->load_unload_ticks = max(1, unloading_time);
-
 	if (completely_emptied) {
 		/* Make sure the vehicle is marked dirty, since we need to update the NewGRF
 		 * properties such as weight, power and TE whenever the trigger runs. */
