@@ -1,4 +1,4 @@
-/* $Id: genworld.h 18861 2010-01-18 14:58:32Z smatz $ */
+/* $Id: genworld.h 23611 2011-12-19 20:56:50Z truebrain $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,17 +15,15 @@
 #include "company_type.h"
 
 /** Constants related to world generation */
-enum {
+enum LandscapeGenerator {
 	/* Order of these enums has to be the same as in lang/english.txt
 	 * Otherwise you will get inconsistent behaviour. */
 	LG_ORIGINAL     = 0,  ///< The original landscape generator
 	LG_TERRAGENESIS = 1,  ///< TerraGenesis Perlin landscape generator
-
-	GENERATE_NEW_SEED = UINT_MAX, ///< Create a new random seed
-
-	GENWORLD_REDRAW_TIMEOUT = 200, ///< Timeout between redraws
 };
 
+static const uint GENERATE_NEW_SEED = UINT_MAX; ///< Create a new random seed
+
 /** Modes for GenerateWorld */
 enum GenWorldMode {
 	GWM_NEWGAME   = 0, ///< Generate a map for a new game
@@ -34,12 +32,15 @@ enum GenWorldMode {
 	GWM_HEIGHTMAP = 3, ///< Generate a newgame from a heightmap
 };
 
+static const uint CUSTOM_SEA_LEVEL_NUMBER_DIFFICULTY = 4; ///< Value for custom sea level in difficulty settings.
+static const uint CUSTOM_SEA_LEVEL_MIN_PERCENTAGE = 1;    ///< Minimum percentage a user can specify for custom sea level.
+static const uint CUSTOM_SEA_LEVEL_MAX_PERCENTAGE = 90;   ///< Maximum percentage a user can specify for custom sea level.
+
 typedef void GWDoneProc();  ///< Procedure called when the genworld process finishes
 typedef void GWAbortProc(); ///< Called when genworld is aborted
 
 /** Properties of current genworld process */
 struct GenWorldInfo {
-	bool active;           ///< Is generating world active
 	bool abort;            ///< Whether to abort the thread ASAP
 	bool quit_thread;      ///< Do we want to quit the active thread
 	bool threaded;         ///< Whether we run _GenerateWorld threaded
@@ -56,27 +57,19 @@ struct GenWorldInfo {
 enum GenWorldProgress {
 	GWP_MAP_INIT,    ///< Initialize/allocate the map, start economy
 	GWP_LANDSCAPE,   ///< Create the landscape
+	GWP_RIVER,       ///< Create the rivers
 	GWP_ROUGH_ROCKY, ///< Make rough and rocky areas
 	GWP_TOWN,        ///< Generate towns
 	GWP_INDUSTRY,    ///< Generate industries
-	GWP_UNMOVABLE,   ///< Generate unmovables (radio tower, light houses)
+	GWP_OBJECT,      ///< Generate objects (radio tower, light houses)
 	GWP_TREE,        ///< Generate trees
 	GWP_GAME_INIT,   ///< Initialize the game
 	GWP_RUNTILELOOP, ///< Runs the tile loop 1280 times to make snow etc
+	GWP_RUNSCRIPT,   ///< Runs the game script at most 2500 times, or when ever the script sleeps
 	GWP_GAME_START,  ///< Really prepare to start the game
 	GWP_CLASS_COUNT
 };
 
-/**
- * Check if we are currently in the process of generating a world.
- * @return are we generating world?
- */
-static inline bool IsGeneratingWorld()
-{
-	extern GenWorldInfo _gw;
-	return _gw.active;
-}
-
 /* genworld.cpp */
 bool IsGenerateWorldThreaded();
 void GenerateWorldSetCallback(GWDoneProc *proc);
@@ -88,6 +81,7 @@ bool IsGeneratingWorldAborted();
 void HandleGeneratingWorldAbortion();
 
 /* genworld_gui.cpp */
+void SetNewLandscapeType(byte landscape);
 void SetGeneratingWorldProgress(GenWorldProgress cls, uint total);
 void IncreaseGeneratingWorldProgress(GenWorldProgress cls);
 void PrepareGenerateWorldProgress();
@@ -96,7 +90,6 @@ void StartNewGameWithoutGUI(uint seed);
 void ShowCreateScenario();
 void StartScenarioEditor();
 
-extern class ThreadMutex *_genworld_mapgen_mutex;
-extern class ThreadMutex *_genworld_paint_mutex;
+extern bool _generating_world;
 
 #endif /* GENWORLD_H */
