@@ -1,4 +1,4 @@
-/* $Id: newgrf_canal.cpp 24693 2012-11-10 20:46:39Z alberth $ */
+/* $Id: newgrf_canal.cpp 26482 2014-04-23 20:13:33Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -13,8 +13,11 @@
 #include "debug.h"
 #include "newgrf_spritegroup.h"
 #include "newgrf_canal.h"
+#include "water.h"
 #include "water_map.h"
 
+#include "safeguards.h"
+
 /** Table of canal 'feature' sprite groups */
 WaterFeature _water_feature[CF_END];
 
@@ -22,7 +25,7 @@ WaterFeature _water_feature[CF_END];
 struct CanalScopeResolver : public ScopeResolver {
 	TileIndex tile; ///< Tile containing the canal.
 
-	CanalScopeResolver(ResolverObject *ro, TileIndex tile);
+	CanalScopeResolver(ResolverObject &ro, TileIndex tile);
 
 	/* virtual */ uint32 GetRandomBits() const;
 	/* virtual */ uint32 GetVariable(byte variable, uint32 parameter, bool *available) const;
@@ -32,7 +35,7 @@ struct CanalScopeResolver : public ScopeResolver {
 struct CanalResolverObject : public ResolverObject {
 	CanalScopeResolver canal_scope;
 
-	CanalResolverObject(const GRFFile *grffile, TileIndex tile,
+	CanalResolverObject(CanalFeature feature, TileIndex tile,
 			CallbackID callback = CBID_NO_CALLBACK, uint32 callback_param1 = 0, uint32 callback_param2 = 0);
 
 	/* virtual */ ScopeResolver *GetScope(VarSpriteGroupScope scope = VSG_SCOPE_SELF, byte relative = 0)
@@ -66,6 +69,29 @@ struct CanalResolverObject : public ResolverObject {
 		/* Terrain type */
 		case 0x81: return GetTerrainType(this->tile);
 
+		/* Dike map: Connectivity info for river and canal tiles
+		 *
+		 * Assignment of bits to directions defined in agreement with
+		 * http://projects.tt-forums.net/projects/ttdpatch/repository/revisions/2367/entry/trunk/patches/water.asm#L879
+		 *         7
+		 *      3     0
+		 *   6     *     4
+		 *      2     1
+		 *         5
+		 */
+		case 0x82: {
+			uint32 connectivity =
+				  (!IsWateredTile(TILE_ADDXY(tile, -1,  0), DIR_SW) << 0)  // NE
+				+ (!IsWateredTile(TILE_ADDXY(tile,  0,  1), DIR_NW) << 1)  // SE
+				+ (!IsWateredTile(TILE_ADDXY(tile,  1,  0), DIR_NE) << 2)  // SW
+				+ (!IsWateredTile(TILE_ADDXY(tile,  0, -1), DIR_SE) << 3)  // NW
+				+ (!IsWateredTile(TILE_ADDXY(tile, -1,  1), DIR_W)  << 4)  // E
+				+ (!IsWateredTile(TILE_ADDXY(tile,  1,  1), DIR_N)  << 5)  // S
+				+ (!IsWateredTile(TILE_ADDXY(tile,  1, -1), DIR_E)  << 6)  // W
+				+ (!IsWateredTile(TILE_ADDXY(tile, -1, -1), DIR_S)  << 7); // N
+			return connectivity;
+		}
+
 		/* Random data for river or canal tiles, otherwise zero */
 		case 0x83: return IsTileType(this->tile, MP_WATER) ? GetWaterTileRandomBits(this->tile) : 0;
 	}
@@ -84,23 +110,24 @@ struct CanalResolverObject : public ResolverObject {
 	return group->loaded[0];
 }
 
-CanalScopeResolver::CanalScopeResolver(ResolverObject *ro, TileIndex tile) : ScopeResolver(ro)
+CanalScopeResolver::CanalScopeResolver(ResolverObject &ro, TileIndex tile) : ScopeResolver(ro)
 {
 	this->tile = tile;
 }
 
 /**
  * Canal resolver constructor.
- * @param grffile Grf file.
+ * @param feature Which canal feature we want.
  * @param tile Tile index of canal.
  * @param callback Callback ID.
  * @param callback_param1 First parameter (var 10) of the callback.
  * @param callback_param2 Second parameter (var 18) of the callback.
  */
-CanalResolverObject::CanalResolverObject(const GRFFile *grffile, TileIndex tile,
+CanalResolverObject::CanalResolverObject(CanalFeature feature, TileIndex tile,
 		CallbackID callback, uint32 callback_param1, uint32 callback_param2)
-		: ResolverObject(grffile, callback, callback_param1, callback_param2), canal_scope(this, tile)
+		: ResolverObject(_water_feature[feature].grffile, callback, callback_param1, callback_param2), canal_scope(*this, tile)
 {
+	this->root_spritegroup = _water_feature[feature].group;
 }
 
 /**
@@ -111,8 +138,8 @@ CanalResolverObject::CanalResolverObject(const GRFFile *grffile, TileIndex tile,
  */
 SpriteID GetCanalSprite(CanalFeature feature, TileIndex tile)
 {
-	CanalResolverObject object(_water_feature[feature].grffile, tile);
-	const SpriteGroup *group = SpriteGroup::Resolve(_water_feature[feature].group, &object);
+	CanalResolverObject object(feature, tile);
+	const SpriteGroup *group = object.Resolve();
 	if (group == NULL) return 0;
 
 	return group->GetResult();
@@ -129,11 +156,8 @@ SpriteID GetCanalSprite(CanalFeature feature, TileIndex tile)
  */
 static uint16 GetCanalCallback(CallbackID callback, uint32 param1, uint32 param2, CanalFeature feature, TileIndex tile)
 {
-	CanalResolverObject object(_water_feature[feature].grffile, tile, callback, param1, param2);
-	const SpriteGroup *group = SpriteGroup::Resolve(_water_feature[feature].group, &object);
-	if (group == NULL) return CALLBACK_FAILED;
-
-	return group->GetCallbackResult();
+	CanalResolverObject object(feature, tile, callback, param1, param2);
+	return object.ResolveCallback();
 }
 
 /**
