@@ -1,4 +1,4 @@
-/* $Id: newgrf_debug_gui.cpp 25989 2013-11-13 21:58:36Z rubidium $ */
+/* $Id: newgrf_debug_gui.cpp 27174 2015-03-01 08:17:14Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -18,6 +18,8 @@
 #include "string_func.h"
 #include "strings_func.h"
 #include "textbuf_gui.h"
+#include "vehicle_gui.h"
+#include "zoom_func.h"
 
 #include "engine_base.h"
 #include "industry.h"
@@ -25,6 +27,8 @@
 #include "station_base.h"
 #include "town.h"
 #include "vehicle_base.h"
+#include "train.h"
+#include "roadveh.h"
 
 #include "newgrf_airporttiles.h"
 #include "newgrf_debug.h"
@@ -40,6 +44,8 @@
 
 #include "table/strings.h"
 
+#include "safeguards.h"
+
 /** The sprite picker. */
 NewGrfDebugSpritePicker _newgrf_debug_sprite_picker = { SPM_NONE, NULL, 0, SmallVector<SpriteID, 256>() };
 
@@ -278,6 +284,9 @@ struct NewGRFInspectWindow : Window {
 	/** GRFID of the caller of this window, 0 if it has no caller. */
 	uint32 caller_grfid;
 
+	/** For ground vehicles: Index in vehicle chain. */
+	uint chain_index;
+
 	/** The currently edited parameter, to update the right one. */
 	byte current_edit_param;
 
@@ -303,31 +312,81 @@ struct NewGRFInspectWindow : Window {
 		this->SetDirty();
 	}
 
-	NewGRFInspectWindow(const WindowDesc *desc, WindowNumber wno) : Window()
+	/**
+	 * Check whether this feature has chain index, i.e. refers to ground vehicles.
+	 */
+	bool HasChainIndex() const
 	{
-		this->CreateNestedTree(desc);
+		GrfSpecFeature f = GetFeatureNum(this->window_number);
+		return f == GSF_TRAINS || f == GSF_ROADVEHICLES;
+	}
+
+	/**
+	 * Get the feature index.
+	 * @return the feature index
+	 */
+	uint GetFeatureIndex() const
+	{
+		uint index = ::GetFeatureIndex(this->window_number);
+		if (this->chain_index > 0) {
+			assert(this->HasChainIndex());
+			const Vehicle *v = Vehicle::Get(index);
+			v = v->Move(this->chain_index);
+			if (v != NULL) index = v->index;
+		}
+		return index;
+	}
+
+	/**
+	 * Ensure that this->chain_index is in range.
+	 */
+	void ValidateChainIndex()
+	{
+		if (this->chain_index == 0) return;
+
+		assert(this->HasChainIndex());
+
+		const Vehicle *v = Vehicle::Get(::GetFeatureIndex(this->window_number));
+		v = v->Move(this->chain_index);
+		if (v == NULL) this->chain_index = 0;
+	}
+
+	NewGRFInspectWindow(WindowDesc *desc, WindowNumber wno) : Window(desc)
+	{
+		this->CreateNestedTree();
 		this->vscroll = this->GetScrollbar(WID_NGRFI_SCROLLBAR);
-		this->FinishInitNested(desc, wno);
+		this->FinishInitNested(wno);
 
 		this->vscroll->SetCount(0);
-		this->SetWidgetDisabledState(WID_NGRFI_PARENT, GetFeatureHelper(this->window_number)->GetParent(GetFeatureIndex(this->window_number)) == UINT32_MAX);
+		this->SetWidgetDisabledState(WID_NGRFI_PARENT, GetFeatureHelper(this->window_number)->GetParent(this->GetFeatureIndex()) == UINT32_MAX);
+
+		this->OnInvalidateData(0, true);
 	}
 
 	virtual void SetStringParameters(int widget) const
 	{
 		if (widget != WID_NGRFI_CAPTION) return;
 
-		GetFeatureHelper(this->window_number)->SetStringParameters(GetFeatureIndex(this->window_number));
+		GetFeatureHelper(this->window_number)->SetStringParameters(this->GetFeatureIndex());
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
-		if (widget != WID_NGRFI_MAINPANEL) return;
+		switch (widget) {
+			case WID_NGRFI_VEH_CHAIN: {
+				assert(this->HasChainIndex());
+				GrfSpecFeature f = GetFeatureNum(this->window_number);
+				size->height = max(size->height, GetVehicleImageCellSize((VehicleType)(VEH_TRAIN + (f - GSF_TRAINS)), EIT_IN_DEPOT).height + 2 + WD_BEVEL_TOP + WD_BEVEL_BOTTOM);
+				break;
+			}
 
-		resize->height = max(11, FONT_HEIGHT_NORMAL + 1);
-		resize->width  = 1;
+			case WID_NGRFI_MAINPANEL:
+				resize->height = max(11, FONT_HEIGHT_NORMAL + 1);
+				resize->width  = 1;
 
-		size->height = 5 * resize->height + TOP_OFFSET + BOTTOM_OFFSET;
+				size->height = 5 * resize->height + TOP_OFFSET + BOTTOM_OFFSET;
+				break;
+		}
 	}
 
 	/**
@@ -342,7 +401,7 @@ struct NewGRFInspectWindow : Window {
 
 		va_list va;
 		va_start(va, format);
-		vsnprintf(buf, lengthof(buf), format, va);
+		vseprintf(buf, lastof(buf), format, va);
 		va_end(va);
 
 		offset -= this->vscroll->GetPosition();
@@ -353,9 +412,47 @@ struct NewGRFInspectWindow : Window {
 
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
+		switch (widget) {
+			case WID_NGRFI_VEH_CHAIN: {
+				const Vehicle *v = Vehicle::Get(this->GetFeatureIndex());
+				int total_width = 0;
+				int sel_start = 0;
+				int sel_end = 0;
+				for (const Vehicle *u = v->First(); u != NULL; u = u->Next()) {
+					if (u == v) sel_start = total_width;
+					switch (u->type) {
+						case VEH_TRAIN: total_width += Train      ::From(u)->GetDisplayImageWidth(); break;
+						case VEH_ROAD:  total_width += RoadVehicle::From(u)->GetDisplayImageWidth(); break;
+						default: NOT_REACHED();
+					}
+					if (u == v) sel_end = total_width;
+				}
+
+				int width = r.right + 1 - r.left - WD_BEVEL_LEFT - WD_BEVEL_RIGHT;
+				int skip = 0;
+				if (total_width > width) {
+					int sel_center = (sel_start + sel_end) / 2;
+					if (sel_center > width / 2) skip = min(total_width - width, sel_center - width / 2);
+				}
+
+				GrfSpecFeature f = GetFeatureNum(this->window_number);
+				int h = GetVehicleImageCellSize((VehicleType)(VEH_TRAIN + (f - GSF_TRAINS)), EIT_IN_DEPOT).height;
+				int y = (r.top + r.bottom - h) / 2;
+				DrawVehicleImage(v->First(), r.left + WD_BEVEL_LEFT, r.right - WD_BEVEL_RIGHT, y + 1, INVALID_VEHICLE, EIT_IN_DETAILS, skip);
+
+				/* Highlight the articulated part (this is different to the whole-vehicle highlighting of DrawVehicleImage */
+				if (_current_text_dir == TD_RTL) {
+					DrawFrameRect(r.right - sel_end   + skip, y, r.right - sel_start + skip, y + h, COLOUR_WHITE, FR_BORDERONLY);
+				} else {
+					DrawFrameRect(r.left  + sel_start - skip, y, r.left  + sel_end   - skip, y + h, COLOUR_WHITE, FR_BORDERONLY);
+				}
+				break;
+			}
+		}
+
 		if (widget != WID_NGRFI_MAINPANEL) return;
 
-		uint index = GetFeatureIndex(this->window_number);
+		uint index = this->GetFeatureIndex();
 		const NIFeature *nif  = GetFeature(this->window_number);
 		const NIHelper *nih   = nif->helper;
 		const void *base      = nih->GetInstance(index);
@@ -458,11 +555,29 @@ struct NewGRFInspectWindow : Window {
 		switch (widget) {
 			case WID_NGRFI_PARENT: {
 				const NIHelper *nih   = GetFeatureHelper(this->window_number);
-				uint index = nih->GetParent(GetFeatureIndex(this->window_number));
-				::ShowNewGRFInspectWindow((GrfSpecFeature)GB(index, 24, 8), GetFeatureIndex(index), nih->GetGRFID(GetFeatureIndex(this->window_number)));
+				uint index = nih->GetParent(this->GetFeatureIndex());
+				::ShowNewGRFInspectWindow(GetFeatureNum(index), ::GetFeatureIndex(index), nih->GetGRFID(this->GetFeatureIndex()));
 				break;
 			}
 
+			case WID_NGRFI_VEH_PREV:
+				if (this->chain_index > 0) {
+					this->chain_index--;
+					this->InvalidateData();
+				}
+				break;
+
+			case WID_NGRFI_VEH_NEXT:
+				if (this->HasChainIndex()) {
+					uint index = this->GetFeatureIndex();
+					Vehicle *v = Vehicle::Get(index);
+					if (v != NULL && v->Next() != NULL) {
+						this->chain_index++;
+						this->InvalidateData();
+					}
+				}
+				break;
+
 			case WID_NGRFI_MAINPANEL: {
 				/* Does this feature have variables? */
 				const NIFeature *nif  = GetFeature(this->window_number);
@@ -497,16 +612,57 @@ struct NewGRFInspectWindow : Window {
 	{
 		this->vscroll->SetCapacityFromWidget(this, WID_NGRFI_MAINPANEL, TOP_OFFSET + BOTTOM_OFFSET);
 	}
+
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
+	{
+		if (!gui_scope) return;
+		if (this->HasChainIndex()) {
+			this->ValidateChainIndex();
+			this->SetWidgetDisabledState(WID_NGRFI_VEH_PREV, this->chain_index == 0);
+			Vehicle *v = Vehicle::Get(this->GetFeatureIndex());
+			this->SetWidgetDisabledState(WID_NGRFI_VEH_NEXT, v == NULL || v->Next() == NULL);
+		}
+	}
 };
 
 /* static */ uint32 NewGRFInspectWindow::var60params[GSF_FAKE_END][0x20] = { {0} }; // Use spec to have 0s in whole array
 
+static const NWidgetPart _nested_newgrf_inspect_chain_widgets[] = {
+	NWidget(NWID_HORIZONTAL),
+		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
+		NWidget(WWT_CAPTION, COLOUR_GREY, WID_NGRFI_CAPTION), SetDataTip(STR_NEWGRF_INSPECT_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
+		NWidget(WWT_SHADEBOX, COLOUR_GREY),
+		NWidget(WWT_DEFSIZEBOX, COLOUR_GREY),
+		NWidget(WWT_STICKYBOX, COLOUR_GREY),
+	EndContainer(),
+	NWidget(WWT_PANEL, COLOUR_GREY),
+		NWidget(NWID_HORIZONTAL),
+			NWidget(WWT_PUSHARROWBTN, COLOUR_GREY, WID_NGRFI_VEH_PREV), SetDataTip(AWV_DECREASE, STR_NULL),
+			NWidget(WWT_PUSHARROWBTN, COLOUR_GREY, WID_NGRFI_VEH_NEXT), SetDataTip(AWV_INCREASE, STR_NULL),
+			NWidget(WWT_EMPTY, COLOUR_GREY, WID_NGRFI_VEH_CHAIN), SetFill(1, 0), SetResize(1, 0),
+		EndContainer(),
+	EndContainer(),
+	NWidget(NWID_HORIZONTAL),
+		NWidget(WWT_PANEL, COLOUR_GREY, WID_NGRFI_MAINPANEL), SetMinimalSize(300, 0), SetScrollbar(WID_NGRFI_SCROLLBAR), EndContainer(),
+		NWidget(NWID_VERTICAL),
+			NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_NGRFI_SCROLLBAR),
+			NWidget(WWT_RESIZEBOX, COLOUR_GREY),
+		EndContainer(),
+	EndContainer(),
+};
+
 static const NWidgetPart _nested_newgrf_inspect_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
 		NWidget(WWT_CAPTION, COLOUR_GREY, WID_NGRFI_CAPTION), SetDataTip(STR_NEWGRF_INSPECT_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_NGRFI_PARENT), SetDataTip(STR_NEWGRF_INSPECT_PARENT_BUTTON, STR_NEWGRF_INSPECT_PARENT_TOOLTIP),
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
+		NWidget(WWT_DEFSIZEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
@@ -518,8 +674,15 @@ static const NWidgetPart _nested_newgrf_inspect_widgets[] = {
 	EndContainer(),
 };
 
-static const WindowDesc _newgrf_inspect_desc(
-	WDP_AUTO, 400, 300,
+static WindowDesc _newgrf_inspect_chain_desc(
+	WDP_AUTO, "newgrf_inspect_chain", 400, 300,
+	WC_NEWGRF_INSPECT, WC_NONE,
+	0,
+	_nested_newgrf_inspect_chain_widgets, lengthof(_nested_newgrf_inspect_chain_widgets)
+);
+
+static WindowDesc _newgrf_inspect_desc(
+	WDP_AUTO, "newgrf_inspect", 400, 300,
 	WC_NEWGRF_INSPECT, WC_NONE,
 	0,
 	_nested_newgrf_inspect_widgets, lengthof(_nested_newgrf_inspect_widgets)
@@ -539,11 +702,27 @@ void ShowNewGRFInspectWindow(GrfSpecFeature feature, uint index, const uint32 gr
 	if (!IsNewGRFInspectable(feature, index)) return;
 
 	WindowNumber wno = GetInspectWindowNumber(feature, index);
-	NewGRFInspectWindow *w = AllocateWindowDescFront<NewGRFInspectWindow>(&_newgrf_inspect_desc, wno);
-	if (w == NULL) w = (NewGRFInspectWindow *)FindWindowById(WC_NEWGRF_INSPECT, wno);
+	WindowDesc *desc = (feature == GSF_TRAINS || feature == GSF_ROADVEHICLES) ? &_newgrf_inspect_chain_desc : &_newgrf_inspect_desc;
+	NewGRFInspectWindow *w = AllocateWindowDescFront<NewGRFInspectWindow>(desc, wno, true);
 	w->SetCallerGRFID(grfid);
 }
 
+/**
+ * Invalidate the inspect window for a given feature and index.
+ * The index is normally an in-game location/identifier, such
+ * as a TileIndex or an IndustryID depending on the feature
+ * we want to inspect.
+ * @param feature The feature we want to invalidate the window for.
+ * @param index   The index/identifier of the feature to invalidate.
+ */
+void InvalidateNewGRFInspectWindow(GrfSpecFeature feature, uint index)
+{
+	if (feature == GSF_INVALID) return;
+
+	WindowNumber wno = GetInspectWindowNumber(feature, index);
+	InvalidateWindowData(WC_NEWGRF_INSPECT, wno);
+}
+
 /**
  * Delete inspect window for a given feature and index.
  * The index is normally an in-game location/identifier, such
@@ -627,14 +806,17 @@ GrfSpecFeature GetGrfSpecFeature(VehicleType type)
 
 /** Window used for aligning sprites. */
 struct SpriteAlignerWindow : Window {
-	SpriteID current_sprite; ///< The currently shown sprite
-	Scrollbar *vscroll;
+	typedef SmallPair<int16, int16> XyOffs;    ///< Pair for x and y offsets of the sprite before alignment. First value contains the x offset, second value y offset.
 
-	SpriteAlignerWindow(const WindowDesc *desc, WindowNumber wno) : Window()
+	SpriteID current_sprite;                   ///< The currently shown sprite.
+	Scrollbar *vscroll;
+	SmallMap<SpriteID, XyOffs> offs_start_map; ///< Mapping of starting offsets for the sprites which have been aligned in the sprite aligner window.
+
+	SpriteAlignerWindow(WindowDesc *desc, WindowNumber wno) : Window(desc)
 	{
-		this->CreateNestedTree(desc);
+		this->CreateNestedTree();
 		this->vscroll = this->GetScrollbar(WID_SA_SCROLLBAR);
-		this->FinishInitNested(desc, wno);
+		this->FinishInitNested(wno);
 
 		/* Oh yes, we assume there is at least one normal sprite! */
 		while (GetSpriteType(this->current_sprite) != ST_NORMAL) this->current_sprite++;
@@ -642,16 +824,30 @@ struct SpriteAlignerWindow : Window {
 
 	virtual void SetStringParameters(int widget) const
 	{
+		const Sprite *spr = GetSprite(this->current_sprite, ST_NORMAL);
 		switch (widget) {
 			case WID_SA_CAPTION:
 				SetDParam(0, this->current_sprite);
 				SetDParamStr(1, FioGetFilename(GetOriginFileSlot(this->current_sprite)));
 				break;
 
-			case WID_SA_OFFSETS: {
-				const Sprite *spr = GetSprite(this->current_sprite, ST_NORMAL);
-				SetDParam(0, spr->x_offs / ZOOM_LVL_BASE);
-				SetDParam(1, spr->y_offs / ZOOM_LVL_BASE);
+			case WID_SA_OFFSETS_ABS:
+				SetDParam(0, spr->x_offs);
+				SetDParam(1, spr->y_offs);
+				break;
+
+			case WID_SA_OFFSETS_REL: {
+				/* Relative offset is new absolute offset - starting absolute offset.
+				 * Show 0, 0 as the relative offsets if entry is not in the map (meaning they have not been changed yet).
+				 */
+				const SmallPair<SpriteID, XyOffs> *key_offs_pair = this->offs_start_map.Find(this->current_sprite);
+				if (key_offs_pair != this->offs_start_map.End()) {
+					SetDParam(0, spr->x_offs - key_offs_pair->second.first);
+					SetDParam(1, spr->y_offs - key_offs_pair->second.second);
+				} else {
+					SetDParam(0, 0);
+					SetDParam(1, 0);
+				}
 				break;
 			}
 
@@ -677,20 +873,20 @@ struct SpriteAlignerWindow : Window {
 			case WID_SA_SPRITE: {
 				/* Center the sprite ourselves */
 				const Sprite *spr = GetSprite(this->current_sprite, ST_NORMAL);
-				int width  = r.right  - r.left + 1;
-				int height = r.bottom - r.top  + 1;
-				int x = r.left - spr->x_offs / ZOOM_LVL_BASE + (width  - spr->width / ZOOM_LVL_BASE) / 2;
-				int y = r.top  - spr->y_offs / ZOOM_LVL_BASE + (height - spr->height / ZOOM_LVL_BASE) / 2;
+				int width  = r.right  - r.left + 1 - WD_BEVEL_LEFT - WD_BEVEL_RIGHT;
+				int height = r.bottom - r.top  + 1 - WD_BEVEL_TOP - WD_BEVEL_BOTTOM;
+				int x = -UnScaleGUI(spr->x_offs) + (width  - UnScaleGUI(spr->width) ) / 2;
+				int y = -UnScaleGUI(spr->y_offs) + (height - UnScaleGUI(spr->height)) / 2;
 
-				/* And draw only the part within the sprite area */
-				SubSprite subspr = {
-					spr->x_offs + (spr->width  - width  * ZOOM_LVL_BASE) / 2 + 1,
-					spr->y_offs + (spr->height - height * ZOOM_LVL_BASE) / 2 + 1,
-					spr->x_offs + (spr->width  + width  * ZOOM_LVL_BASE) / 2 - 1,
-					spr->y_offs + (spr->height + height * ZOOM_LVL_BASE) / 2 - 1,
-				};
+				DrawPixelInfo new_dpi;
+				if (!FillDrawPixelInfo(&new_dpi, r.left + WD_BEVEL_LEFT, r.top + WD_BEVEL_TOP, width, height)) break;
+				DrawPixelInfo *old_dpi = _cur_dpi;
+				_cur_dpi = &new_dpi;
+
+				DrawSprite(this->current_sprite, PAL_NONE, x, y, NULL, ZOOM_LVL_GUI);
+
+				_cur_dpi = old_dpi;
 
-				DrawSprite(this->current_sprite, PAL_NONE, x, y, &subspr, ZOOM_LVL_GUI);
 				break;
 			}
 
@@ -770,17 +966,28 @@ struct SpriteAlignerWindow : Window {
 				 * particular NewGRF developer.
 				 */
 				Sprite *spr = const_cast<Sprite *>(GetSprite(this->current_sprite, ST_NORMAL));
+
+				/* Remember the original offsets of the current sprite, if not already in mapping. */
+				if (!(this->offs_start_map.Contains(this->current_sprite))) {
+					this->offs_start_map.Insert(this->current_sprite, XyOffs(spr->x_offs, spr->y_offs));
+				}
 				switch (widget) {
-					case WID_SA_UP:    spr->y_offs -= ZOOM_LVL_BASE; break;
-					case WID_SA_DOWN:  spr->y_offs += ZOOM_LVL_BASE; break;
-					case WID_SA_LEFT:  spr->x_offs -= ZOOM_LVL_BASE; break;
-					case WID_SA_RIGHT: spr->x_offs += ZOOM_LVL_BASE; break;
+					case WID_SA_UP:    spr->y_offs--; break;
+					case WID_SA_DOWN:  spr->y_offs++; break;
+					case WID_SA_LEFT:  spr->x_offs--; break;
+					case WID_SA_RIGHT: spr->x_offs++; break;
 				}
 				/* Of course, we need to redraw the sprite, but where is it used?
 				 * Everywhere is a safe bet. */
 				MarkWholeScreenDirty();
 				break;
 			}
+
+			case WID_SA_RESET_REL:
+				/* Reset the starting offsets for the current sprite. */
+				this->offs_start_map.Erase(this->current_sprite);
+				this->SetDirty();
+				break;
 		}
 	}
 
@@ -814,7 +1021,6 @@ struct SpriteAlignerWindow : Window {
 	virtual void OnResize()
 	{
 		this->vscroll->SetCapacityFromWidget(this, WID_SA_LIST);
-		this->GetWidget<NWidgetCore>(WID_SA_LIST)->widget_data = (this->vscroll->GetCapacity() << MAT_ROW_START) + (1 << MAT_COL_START);
 	}
 };
 
@@ -857,14 +1063,18 @@ static const NWidgetPart _nested_sprite_aligner_widgets[] = {
 					NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_SA_DOWN), SetDataTip(SPR_ARROW_DOWN, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
 					NWidget(NWID_SPACER), SetFill(1, 1),
 				EndContainer(),
+				NWidget(WWT_LABEL, COLOUR_GREY, WID_SA_OFFSETS_ABS), SetDataTip(STR_SPRITE_ALIGNER_OFFSETS_ABS, STR_NULL), SetFill(1, 0), SetPadding(0, 10, 0, 10),
+				NWidget(WWT_LABEL, COLOUR_GREY, WID_SA_OFFSETS_REL), SetDataTip(STR_SPRITE_ALIGNER_OFFSETS_REL, STR_NULL), SetFill(1, 0), SetPadding(0, 10, 0, 10),
 				NWidget(NWID_HORIZONTAL), SetPIP(10, 5, 10),
-					NWidget(WWT_LABEL, COLOUR_GREY, WID_SA_OFFSETS), SetDataTip(STR_SPRITE_ALIGNER_OFFSETS, STR_NULL), SetFill(1, 0),
+					NWidget(NWID_SPACER), SetFill(1, 1),
+					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_SA_RESET_REL), SetDataTip(STR_SPRITE_ALIGNER_RESET_BUTTON, STR_SPRITE_ALIGNER_RESET_TOOLTIP), SetFill(0, 0),
+					NWidget(NWID_SPACER), SetFill(1, 1),
 				EndContainer(),
 			EndContainer(),
 			NWidget(NWID_VERTICAL), SetPIP(10, 5, 10),
 				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_SA_PICKER), SetDataTip(STR_SPRITE_ALIGNER_PICKER_BUTTON, STR_SPRITE_ALIGNER_PICKER_TOOLTIP), SetFill(1, 0),
 				NWidget(NWID_HORIZONTAL),
-					NWidget(WWT_MATRIX, COLOUR_GREY, WID_SA_LIST), SetResize(1, 1), SetDataTip(0x101, STR_NULL), SetFill(1, 1), SetScrollbar(WID_SA_SCROLLBAR),
+					NWidget(WWT_MATRIX, COLOUR_GREY, WID_SA_LIST), SetResize(1, 1), SetMatrixDataTip(1, 0, STR_NULL), SetFill(1, 1), SetScrollbar(WID_SA_SCROLLBAR),
 					NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_SA_SCROLLBAR),
 				EndContainer(),
 			EndContainer(),
@@ -872,8 +1082,8 @@ static const NWidgetPart _nested_sprite_aligner_widgets[] = {
 	EndContainer(),
 };
 
-static const WindowDesc _sprite_aligner_desc(
-	WDP_AUTO, 400, 300,
+static WindowDesc _sprite_aligner_desc(
+	WDP_AUTO, "sprite_aligner", 400, 300,
 	WC_SPRITE_ALIGNER, WC_NONE,
 	0,
 	_nested_sprite_aligner_widgets, lengthof(_nested_sprite_aligner_widgets)
