@@ -1,4 +1,4 @@
-/* $Id: newgrf_station.cpp 26595 2014-05-18 11:21:59Z frosch $ */
+/* $Id: newgrf_station.cpp 26580 2014-05-11 18:02:11Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -26,6 +26,8 @@
 #include "newgrf_animation_base.h"
 #include "newgrf_class_func.h"
 
+#include "safeguards.h"
+
 
 template <typename Tspec, typename Tid, Tid Tmax>
 /* static */ void NewGRFClass<Tspec, Tid, Tmax>::InsertDefaults()
@@ -545,7 +547,7 @@ uint32 Waypoint::GetNewGRFVariable(const ResolverObject &object, byte variable,
  */
 StationResolverObject::StationResolverObject(const StationSpec *statspec, BaseStation *st, TileIndex tile,
 		CallbackID callback, uint32 callback_param1, uint32 callback_param2)
-	: ResolverObject((statspec != NULL ? statspec->grf_prop.grffile : NULL), callback, callback_param1, callback_param2),
+	: ResolverObject(statspec->grf_prop.grffile, callback, callback_param1, callback_param2),
 	station_scope(*this, statspec, st, tile), town_scope(NULL)
 {
 	/* Invalidate all cached vars */
