@@ -1,4 +1,4 @@
-/* $Id: news_type.h 19507 2010-03-23 22:25:43Z smatz $ */
+/* $Id: news_type.h 24843 2012-12-23 21:07:12Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,12 +17,25 @@
 #include "strings_type.h"
 #include "sound_type.h"
 
+/** Constants in the message options window. */
+enum MessageOptionsSpace {
+	MOS_WIDG_PER_SETTING      = 4,  ///< Number of widgets needed for each news category, starting at widget #WID_MO_START_OPTION.
+
+	MOS_LEFT_EDGE             = 6,  ///< Number of pixels between left edge of the window and the options buttons column.
+	MOS_COLUMN_SPACING        = 4,  ///< Number of pixels between the buttons and the description columns.
+	MOS_RIGHT_EDGE            = 6,  ///< Number of pixels between right edge of the window and the options descriptions column.
+	MOS_BUTTON_SPACE          = 10, ///< Additional space in the button with the option value (for better looks).
+
+	MOS_ABOVE_GLOBAL_SETTINGS = 6,  ///< Number of vertical pixels between the categories and the global options.
+	MOS_BOTTOM_EDGE           = 6,  ///< Number of pixels between bottom edge of the window and bottom of the global options.
+};
+
 /**
  * Type of news.
  */
 enum NewsType {
-	NT_ARRIVAL_COMPANY, ///< Cargo arrived for company
-	NT_ARRIVAL_OTHER,   ///< Cargo arrived for competitor
+	NT_ARRIVAL_COMPANY, ///< First vehicle arrived for company
+	NT_ARRIVAL_OTHER,   ///< First vehicle arrived for competitor
 	NT_ACCIDENT,        ///< An accident or disaster has occurred
 	NT_COMPANY_INFO,    ///< Company info (new companies, bankruptcy messages)
 	NT_INDUSTRY_OPEN,   ///< Opening of industries
@@ -39,31 +52,6 @@ enum NewsType {
 	NT_END,             ///< end-of-array marker
 };
 
-/**
- * News subtypes.
- */
-enum NewsSubtype {
-	NS_ARRIVAL_COMPANY,  ///< NT_ARRIVAL_COMPANY
-	NS_ARRIVAL_OTHER,    ///< NT_ARRIVAL_OTHER
-	NS_ACCIDENT,         ///< NT_ACCIDENT
-	NS_COMPANY_TROUBLE,  ///< NT_COMPANY_INFO (trouble)
-	NS_COMPANY_MERGER,   ///< NT_COMPANY_INFO (merger)
-	NS_COMPANY_BANKRUPT, ///< NT_COMPANY_INFO (bankrupt)
-	NS_COMPANY_NEW,      ///< NT_COMPANY_INFO (new company)
-	NS_INDUSTRY_OPEN,    ///< NT_INDUSTRY_OPEN
-	NS_INDUSTRY_CLOSE,   ///< NT_INDUSTRY_CLOSE
-	NS_ECONOMY,          ///< NT_ECONOMY
-	NS_INDUSTRY_COMPANY, ///< NT_INDUSTRY_COMPANY
-	NS_INDUSTRY_OTHER,   ///< NT_INDUSTRY_OTHER
-	NS_INDUSTRY_NOBODY,  ///< NT_INDUSTRY_NOBODY
-	NS_ADVICE,           ///< NT_ADVICE
-	NS_NEW_VEHICLES,     ///< NT_NEW_VEHICLES
-	NS_ACCEPTANCE,       ///< NT_ACCEPTANCE
-	NS_SUBSIDIES,        ///< NT_SUBSIDIES
-	NS_GENERAL,          ///< NT_GENERAL
-	NS_END,              ///< end-of-array marker
-};
-
 /**
  * References to objects in news.
  *
@@ -80,7 +68,7 @@ enum NewsReferenceType {
 	NR_STATION,   ///< Reference station.  Scroll to station when clicking on the news. Delete news when station is deleted.
 	NR_INDUSTRY,  ///< Reference industry. Scroll to industry when clicking on the news. Delete news when industry is deleted.
 	NR_TOWN,      ///< Reference town.     Scroll to town when clicking on the news.
-	NR_ENGINE     ///< Reference engine.
+	NR_ENGINE,    ///< Reference engine.
 };
 
 /**
@@ -88,14 +76,23 @@ enum NewsReferenceType {
  * @note #NF_INCOLOUR is set automatically if needed.
  */
 enum NewsFlag {
-	NFB_INCOLOUR       = 0, ///< News item is shown in colour (otherwise it is shown in black & white).
-	NFB_NO_TRANSPARENT = 1, ///< News item disables transparency in the viewport.
-	NFB_SHADE          = 2, ///< News item uses shaded colours.
+	NFB_INCOLOUR       = 0,                      ///< News item is shown in colour (otherwise it is shown in black & white).
+	NFB_NO_TRANSPARENT = 1,                      ///< News item disables transparency in the viewport.
+	NFB_SHADE          = 2,                      ///< News item uses shaded colours.
+	NFB_WINDOW_LAYOUT  = 3,                      ///< First bit for window layout.
+	NFB_WINDOW_LAYOUT_COUNT = 3,                 ///< Number of bits for window layout.
+	NFB_VEHICLE_PARAM0 = 6,                      ///< String param 0 contains a vehicle ID. (special autoreplace behaviour)
 
-	NF_NONE           = 0,      ///< No flag is set.
-	NF_INCOLOUR       = 1 << 0, ///< Bit value for coloured news.
-	NF_NO_TRANSPARENT = 1 << 1, ///< Bit value for disabling transparency.
-	NF_SHADE          = 1 << 2, ///< Bit value for enabling shading.
+	NF_INCOLOUR       = 1 << NFB_INCOLOUR,       ///< Bit value for coloured news.
+	NF_NO_TRANSPARENT = 1 << NFB_NO_TRANSPARENT, ///< Bit value for disabling transparency.
+	NF_SHADE          = 1 << NFB_SHADE,          ///< Bit value for enabling shading.
+	NF_VEHICLE_PARAM0 = 1 << NFB_VEHICLE_PARAM0, ///< Bit value for specifying that string param 0 contains a vehicle ID. (special autoreplace behaviour)
+
+	NF_THIN           = 0 << NFB_WINDOW_LAYOUT,  ///< Thin news item. (Newspaper with headline and viewport)
+	NF_SMALL          = 1 << NFB_WINDOW_LAYOUT,  ///< Small news item. (Information window with text and viewport)
+	NF_NORMAL         = 2 << NFB_WINDOW_LAYOUT,  ///< Normal news item. (Newspaper with text only)
+	NF_VEHICLE        = 3 << NFB_WINDOW_LAYOUT,  ///< Vehicle news item. (new engine available)
+	NF_COMPANY        = 4 << NFB_WINDOW_LAYOUT,  ///< Company news item. (Newspaper with face)
 };
 DECLARE_ENUM_AS_BIT_SET(NewsFlag)
 
@@ -116,25 +113,30 @@ struct NewsTypeData {
 	const char * const name;    ///< Name
 	const byte age;             ///< Maximum age of news items (in days)
 	const SoundFx sound;        ///< Sound
-	NewsDisplay display;        ///< Display mode (off, summary, full)
-	const StringID description; ///< Description of the news type in news settings window
 
-	NewsTypeData(const char *name, byte age, SoundFx sound, StringID description) :
+	/**
+	 * Construct this entry.
+	 * @param name The name of the type.
+	 * @param age The maximum age for these messages.
+	 * @param sound The sound to play.
+	 */
+	NewsTypeData(const char *name, byte age, SoundFx sound) :
 		name(name),
 		age(age),
-		sound(sound),
-		display(ND_FULL),
-		description(description)
+		sound(sound)
 	{
 	}
+
+	NewsDisplay GetDisplay() const;
 };
 
+/** Information about a single item of news. */
 struct NewsItem {
 	NewsItem *prev;              ///< Previous news item
 	NewsItem *next;              ///< Next news item
 	StringID string_id;          ///< Message text
 	Date date;                   ///< Date of the news
-	NewsSubtype subtype;         ///< News subtype @see NewsSubtype
+	NewsType type;               ///< Type of the news
 	NewsFlag flags;              ///< NewsFlags bits @see NewsFlag
 
 	NewsReferenceType reftype1;  ///< Type of ref1
@@ -149,7 +151,7 @@ struct NewsItem {
 		free(this->free_data);
 	}
 
-	uint64 params[10];
+	uint64 params[10]; ///< Parameters for string resolving.
 };
 
 /**
