@@ -1,4 +1,4 @@
-/* $Id: unix.cpp 22021 2011-02-07 22:49:38Z rubidium $ */
+/* $Id: unix.cpp 26724 2014-08-09 19:39:14Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,6 +14,8 @@
 #include "../../openttd.h"
 #include "../../crashlog.h"
 #include "../../core/random_func.hpp"
+#include "../../debug.h"
+#include "../../string_func.h"
 
 
 #include <dirent.h>
@@ -28,10 +30,18 @@
 	#define HAS_STATVFS
 #endif
 
+#if defined(OPENBSD) || defined(__NetBSD__) || defined(__FreeBSD__)
+	#define HAS_SYSCTL
+#endif
+
 #ifdef HAS_STATVFS
 #include <sys/statvfs.h>
 #endif
 
+#ifdef HAS_SYSCTL
+#include <sys/sysctl.h>
+#endif
+
 
 #ifdef __MORPHOS__
 #include <exec/types.h>
@@ -54,6 +64,8 @@ ULONG __stack = (1024*1024)*2; // maybe not that much is needed actually ;)
 	#endif
 #endif
 
+#include "../../safeguards.h"
+
 bool FiosIsRoot(const char *path)
 {
 #if !defined(__MORPHOS__) && !defined(__AMIGAOS__)
@@ -96,13 +108,13 @@ bool FiosIsValidFile(const char *path, const struct dirent *ent, struct stat *sb
 #if defined(__MORPHOS__) || defined(__AMIGAOS__)
 	/* On MorphOS or AmigaOS paths look like: "Volume:directory/subdirectory" */
 	if (FiosIsRoot(path)) {
-		res = snprintf(filename, lengthof(filename), "%s:%s", path, ent->d_name);
+		res = seprintf(filename, lastof(filename), "%s:%s", path, ent->d_name);
 	} else // XXX - only next line!
 #else
 	assert(path[strlen(path) - 1] == PATHSEPCHAR);
 	if (strlen(path) > 2) assert(path[strlen(path) - 2] != PATHSEPCHAR);
 #endif
-	res = snprintf(filename, lengthof(filename), "%s%s", path, ent->d_name);
+	res = seprintf(filename, lastof(filename), "%s%s", path, ent->d_name);
 
 	/* Could we fully concatenate the path and filename? */
 	if (res >= (int)lengthof(filename) || res < 0) return false;
@@ -263,7 +275,7 @@ int CDECL main(int argc, char *argv[])
 
 	signal(SIGPIPE, SIG_IGN);
 
-	ret = ttd_main(argc, argv);
+	ret = openttd_main(argc, argv);
 
 #ifdef WITH_COCOA
 	cocoaReleaseAutoreleasePool();
@@ -273,7 +285,7 @@ int CDECL main(int argc, char *argv[])
 }
 
 #ifndef WITH_COCOA
-bool GetClipboardContents(char *buffer, size_t buff_len)
+bool GetClipboardContents(char *buffer, const char *last)
 {
 	return false;
 }
@@ -318,3 +330,49 @@ void CSleep(int milliseconds)
 		usleep(milliseconds * 1000);
 	#endif
 }
+
+
+#ifndef __APPLE__
+uint GetCPUCoreCount()
+{
+	uint count = 1;
+#ifdef HAS_SYSCTL
+	int ncpu = 0;
+	size_t len = sizeof(ncpu);
+
+#ifdef OPENBSD
+	int name[2];
+	name[0] = CTL_HW;
+	name[1] = HW_NCPU;
+	if (sysctl(name, 2, &ncpu, &len, NULL, 0) < 0) {
+	        ncpu = 0;
+	}
+#else
+	if (sysctlbyname("hw.availcpu", &ncpu, &len, NULL, 0) < 0) {
+		sysctlbyname("hw.ncpu", &ncpu, &len, NULL, 0);
+	}
+#endif /* #ifdef OPENBSD */
+
+	if (ncpu > 0) count = ncpu;
+#elif defined(_SC_NPROCESSORS_ONLN)
+	long res = sysconf(_SC_NPROCESSORS_ONLN);
+	if (res > 0) count = res;
+#endif
+
+	return count;
+}
+
+void OSOpenBrowser(const char *url)
+{
+	pid_t child_pid = fork();
+	if (child_pid != 0) return;
+
+	const char *args[3];
+	args[0] = "xdg-open";
+	args[1] = url;
+	args[2] = NULL;
+	execvp(args[0], const_cast<char * const *>(args));
+	DEBUG(misc, 0, "Failed to open url: %s", url);
+	exit(0);
+}
+#endif
