@@ -1,4 +1,4 @@
-/* $Id: rail.h 20095 2010-07-08 19:59:13Z rubidium $ */
+/* $Id: rail.h 27432 2015-11-01 12:03:13Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,7 +19,10 @@
 #include "economy_func.h"
 #include "slope_type.h"
 #include "strings_type.h"
+#include "date_type.h"
+#include "signal_type.h"
 
+/** Railtype flags. */
 enum RailTypeFlags {
 	RTF_CATENARY          = 0,                           ///< Bit number for drawing a catenary.
 	RTF_NO_LEVEL_CROSSING = 1,                           ///< Bit number for disallowing level crossings.
@@ -28,10 +31,11 @@ enum RailTypeFlags {
 	RTFB_CATENARY          = 1 << RTF_CATENARY,          ///< Value for drawing a catenary.
 	RTFB_NO_LEVEL_CROSSING = 1 << RTF_NO_LEVEL_CROSSING, ///< Value for disallowing level crossings.
 };
-DECLARE_ENUM_AS_BIT_SET(RailTypeFlags);
+DECLARE_ENUM_AS_BIT_SET(RailTypeFlags)
 
 struct SpriteGroup;
 
+/** Sprite groups for a railtype. */
 enum RailTypeSpriteGroup {
 	RTSG_CURSORS,     ///< Cursor and toolbar icon images
 	RTSG_OVERLAY,     ///< Images for overlaying track
@@ -43,6 +47,8 @@ enum RailTypeSpriteGroup {
 	RTSG_CROSSING,    ///< Level crossing overlay images
 	RTSG_DEPOT,       ///< Depot images
 	RTSG_FENCES,      ///< Fence images
+	RTSG_TUNNEL_PORTAL, ///< Tunnel portal overlay
+	RTSG_SIGNALS,     ///< Signal images
 	RTSG_END,
 };
 
@@ -70,7 +76,7 @@ enum RailTrackOffset {
 };
 
 /**
- * Offsets for spries within a bridge surface overlay set.
+ * Offsets for sprites within a bridge surface overlay set.
  */
 enum RailTrackBridgeOffset {
 	RTBO_X,     ///< Piece of rail in X direction
@@ -78,7 +84,8 @@ enum RailTrackBridgeOffset {
 	RTBO_SLOPE, ///< Sloped rail pieces, in order NE, SE, SW, NW
 };
 
-/** Offsets from base sprite for fence sprites. These are in the order of
+/**
+ * Offsets from base sprite for fence sprites. These are in the order of
  *  the sprites in the original data files.
  */
 enum RailFenceOffset {
@@ -92,11 +99,17 @@ enum RailFenceOffset {
 	RFO_SLOPE_NW,
 };
 
-/** This struct contains all the info that is needed to draw and construct tracks.
+/** List of rail type labels. */
+typedef SmallVector<RailTypeLabel, 4> RailTypeLabelList;
+
+/**
+ * This struct contains all the info that is needed to draw and construct tracks.
  */
 struct RailtypeInfo {
-	/** Struct containing the main sprites. @note not all sprites are listed, but only
-	 *  the ones used directly in the code */
+	/**
+	 * Struct containing the main sprites. @note not all sprites are listed, but only
+	 *  the ones used directly in the code
+	 */
 	struct {
 		SpriteID track_y;      ///< single piece of rail in Y direction, with ground
 		SpriteID track_ns;     ///< two pieces of rail in North and South corner (East-West direction)
@@ -107,13 +120,15 @@ struct RailtypeInfo {
 		SpriteID single_s;     ///< single piece of rail in the southern corner
 		SpriteID single_e;     ///< single piece of rail in the eastern corner
 		SpriteID single_w;     ///< single piece of rail in the western corner
-		SpriteID single_sloped;///< single piecs of rail for slopes
+		SpriteID single_sloped;///< single piece of rail for slopes
 		SpriteID crossing;     ///< level crossing, rail in X direction
 		SpriteID tunnel;       ///< tunnel sprites base
 	} base_sprites;
 
-	/** struct containing the sprites for the rail GUI. @note only sprites referred to
-	 * directly in the code are listed */
+	/**
+	 * struct containing the sprites for the rail GUI. @note only sprites referred to
+	 * directly in the code are listed
+	 */
 	struct {
 		SpriteID build_ns_rail;      ///< button for building single rail in N-S direction
 		SpriteID build_x_rail;       ///< button for building single rail in X direction
@@ -123,6 +138,7 @@ struct RailtypeInfo {
 		SpriteID build_depot;        ///< button for building depots
 		SpriteID build_tunnel;       ///< button for building a tunnel
 		SpriteID convert_rail;       ///< button for converting rail
+		SpriteID signals[SIGTYPE_END][2][2]; ///< signal GUI sprites (type, variant, state)
 	} gui_sprites;
 
 	struct {
@@ -134,15 +150,16 @@ struct RailtypeInfo {
 		CursorID depot;      ///< Cursor for building a depot
 		CursorID tunnel;     ///< Cursor for building a tunnel
 		CursorID convert;    ///< Cursor for converting track
-	} cursor;
+	} cursor;                    ///< Cursors associated with the rail type.
 
 	struct {
-		StringID toolbar_caption;
-		StringID menu_text;
-		StringID build_caption;
-		StringID replace_text;
-		StringID new_loco;
-	} strings;
+		StringID name;            ///< Name of this rail type.
+		StringID toolbar_caption; ///< Caption in the construction toolbar GUI for this rail type.
+		StringID menu_text;       ///< Name of this rail type in the main toolbar dropdown.
+		StringID build_caption;   ///< Caption of the build vehicle GUI for this rail type.
+		StringID replace_text;    ///< Text used in the autoreplace GUI.
+		StringID new_loco;        ///< Name of an engine for this type of rail in the engine preview GUI.
+	} strings;                        ///< Strings associated with the rail type.
 
 	/** sprite number difference between a piece of track on a snowy ground and the corresponding one on normal ground */
 	SpriteID snow_offset;
@@ -153,25 +170,15 @@ struct RailtypeInfo {
 	/** bitmask to the OTHER railtypes on which an engine of THIS railtype can physically travel */
 	RailTypes compatible_railtypes;
 
-	/**
-	 * Offset between the current railtype and normal rail. This means that:<p>
-	 * 1) All the sprites in a railset MUST be in the same order. This order
-	 *    is determined by normal rail. Check sprites 1005 and following for this order<p>
-	 * 2) The position where the railtype is loaded must always be the same, otherwise
-	 *    the offset will fail.
-	 * @note: Something more flexible might be desirable in the future.
-	 */
-	SpriteID total_offset;
-
 	/**
 	 * Bridge offset
 	 */
 	SpriteID bridge_offset;
 
 	/**
-	 * Offset to add to ground sprite when drawing custom waypoints / stations
+	 * Original railtype number to use when drawing non-newgrf railtypes, or when drawing stations.
 	 */
-	byte custom_ground_offset;
+	byte fallback_railtype;
 
 	/**
 	 * Multiplier for curve maximum speed advantage
@@ -188,6 +195,11 @@ struct RailtypeInfo {
 	 */
 	uint16 cost_multiplier;
 
+	/**
+	 * Cost multiplier for maintenance of this rail type
+	 */
+	uint16 maintenance_multiplier;
+
 	/**
 	 * Acceleration type of this rail type
 	 */
@@ -203,11 +215,46 @@ struct RailtypeInfo {
 	 */
 	RailTypeLabel label;
 
+	/**
+	 * Rail type labels this type provides in addition to the main label.
+	 */
+	RailTypeLabelList alternate_labels;
+
 	/**
 	 * Colour on mini-map
 	 */
 	byte map_colour;
 
+	/**
+	 * Introduction date.
+	 * When #INVALID_DATE or a vehicle using this railtype gets introduced earlier,
+	 * the vehicle's introduction date will be used instead for this railtype.
+	 * The introduction at this date is furthermore limited by the
+	 * #introduction_required_types.
+	 */
+	Date introduction_date;
+
+	/**
+	 * Bitmask of railtypes that are required for this railtype to be introduced
+	 * at a given #introduction_date.
+	 */
+	RailTypes introduction_required_railtypes;
+
+	/**
+	 * Bitmask of which other railtypes are introduced when this railtype is introduced.
+	 */
+	RailTypes introduces_railtypes;
+
+	/**
+	 * The sorting order of this railtype for the toolbar dropdown.
+	 */
+	byte sorting_order;
+
+	/**
+	 * NewGRF providing the Action3 for the railtype. NULL if not available.
+	 */
+	const GRFFile *grffile[RTSG_END];
+
 	/**
 	 * Sprite groups for resolving sprites
 	 */
@@ -217,6 +264,18 @@ struct RailtypeInfo {
 	{
 		return this->group[RTSG_GROUND] != NULL;
 	}
+
+	/**
+	 * Offset between the current railtype and normal rail. This means that:<p>
+	 * 1) All the sprites in a railset MUST be in the same order. This order
+	 *    is determined by normal rail. Check sprites 1005 and following for this order<p>
+	 * 2) The position where the railtype is loaded must always be the same, otherwise
+	 *    the offset will fail.
+	 */
+	inline uint GetRailtypeSpriteOffset() const
+	{
+		return 82 * this->fallback_railtype;
+	}
 };
 
 
@@ -303,86 +362,74 @@ static inline Money RailClearCost(RailType railtype)
  */
 static inline Money RailConvertCost(RailType from, RailType to)
 {
-	/* rail -> el. rail
-	 * calculate the price as 5 / 4 of (cost build el. rail) - (cost build rail)
-	 * (the price of workers to get to place is that 1/4)
-	 */
-	if (HasPowerOnRail(from, to)) {
-		Money cost = ((RailBuildCost(to) - RailBuildCost(from)) * 5) >> 2;
-		if (cost != 0) return cost;
+	/* Get the costs for removing and building anew
+	 * A conversion can never be more costly */
+	Money rebuildcost = RailBuildCost(to) + RailClearCost(from);
+
+	/* Conversion between somewhat compatible railtypes:
+	 * Pay 1/8 of the target rail cost (labour costs) and additionally any difference in the
+	 * build costs, if the target type is more expensive (material upgrade costs).
+	 * Upgrade can never be more expensive than re-building. */
+	if (HasPowerOnRail(from, to) || HasPowerOnRail(to, from)) {
+		Money upgradecost = RailBuildCost(to) / 8 + max((Money)0, RailBuildCost(to) - RailBuildCost(from));
+		return min(upgradecost, rebuildcost);
 	}
 
-	/* el. rail -> rail
-	 * calculate the price as 1 / 4 of (cost build el. rail) - (cost build rail)
-	 * (the price of workers is 1 / 4 + price of copper sold to a recycle center)
-	 */
-	if (HasPowerOnRail(to, from)) {
-		Money cost = (RailBuildCost(from) - RailBuildCost(to)) >> 2;
-		if (cost != 0) return cost;
-	}
+	/* make the price the same as remove + build new type for rail types
+	 * which are not compatible in any way */
+	return rebuildcost;
+}
 
-	/* make the price the same as remove + build new type */
-	return RailBuildCost(to) + RailClearCost(from);
+/**
+ * Calculates the maintenance cost of a number of track bits.
+ * @param railtype The railtype to get the cost of.
+ * @param num Number of track bits of this railtype.
+ * @param total_num Total number of track bits of all railtypes.
+ * @return Total cost.
+ */
+static inline Money RailMaintenanceCost(RailType railtype, uint32 num, uint32 total_num)
+{
+	assert(railtype < RAILTYPE_END);
+	return (_price[PR_INFRASTRUCTURE_RAIL] * GetRailTypeInfo(railtype)->maintenance_multiplier * num * (1 + IntSqrt(total_num))) >> 11; // 4 bits fraction for the multiplier and 7 bits scaling.
+}
+
+/**
+ * Calculates the maintenance cost of a number of signals.
+ * @param num Number of signals.
+ * @return Total cost.
+ */
+static inline Money SignalMaintenanceCost(uint32 num)
+{
+	return (_price[PR_INFRASTRUCTURE_RAIL] * 15 * num * (1 + IntSqrt(num))) >> 8; // 1 bit fraction for the multiplier and 7 bits scaling.
 }
 
 void DrawTrainDepotSprite(int x, int y, int image, RailType railtype);
-Vehicle *EnsureNoTrainOnTrackProc(Vehicle *v, void *data);
 int TicksToLeaveDepot(const Train *v);
 
 Foundation GetRailFoundation(Slope tileh, TrackBits bits);
 
 
-/**
- * Finds out if a company has a certain railtype available
- * @param company the company in question
- * @param railtype requested RailType
- * @return true if company has requested RailType available
- */
 bool HasRailtypeAvail(const CompanyID company, const RailType railtype);
-
-/**
- * Validate functions for rail building.
- * @param rail the railtype to check.
- * @return true if the current company may build the rail.
- */
 bool ValParamRailtype(const RailType rail);
 
-/**
- * Returns the "best" railtype a company can build.
- * As the AI doesn't know what the BEST one is, we have our own priority list
- * here. When adding new railtypes, modify this function
- * @param company the company "in action"
- * @return The "best" railtype a company has available
- */
-RailType GetBestRailtype(const CompanyID company);
+RailTypes AddDateIntroducedRailTypes(RailTypes current, Date date);
 
-/**
- * Get the rail types the given company can build.
- * @param c the company to get the rail types for.
- * @return the rail types.
- */
+RailType GetBestRailtype(const CompanyID company);
 RailTypes GetCompanyRailtypes(const CompanyID c);
 
-/**
- * Get the rail type for a given label.
- * @param label the railtype label.
- * @return the railtype.
- */
-RailType GetRailTypeByLabel(RailTypeLabel label);
+RailType GetRailTypeByLabel(RailTypeLabel label, bool allow_alternate_labels = true);
 
-/**
- * Reset all rail type information to its default values.
- */
 void ResetRailTypes();
-
-/**
- * Resolve sprites of custom rail types
- */
 void InitRailTypes();
-
-/**
- * Allocate a new rail type label
- */
 RailType AllocateRailType(RailTypeLabel label);
 
+extern RailType _sorted_railtypes[RAILTYPE_END];
+extern uint8 _sorted_railtypes_size;
+
+/**
+ * Loop header for iterating over railtypes, sorted by sortorder.
+ * @param var Railtype.
+ */
+#define FOR_ALL_SORTED_RAILTYPES(var) for (uint8 index = 0; index < _sorted_railtypes_size && (var = _sorted_railtypes[index], true) ; index++)
+
 #endif /* RAIL_H */
