@@ -1,4 +1,4 @@
-/* $Id: oldloader.h 20430 2010-08-09 21:48:19Z rubidium $ */
+/* $Id: oldloader.h 23595 2011-12-19 17:48:04Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,10 +15,8 @@
 #include "saveload.h"
 #include "../tile_type.h"
 
-enum {
-	BUFFER_SIZE = 4096,
-	OLD_MAP_SIZE = 256 * 256,
-};
+static const uint BUFFER_SIZE = 4096;
+static const uint OLD_MAP_SIZE = 256 * 256;
 
 struct LoadgameState {
 	FILE *file;
@@ -80,10 +78,10 @@ enum OldChunkType {
 	 */
 	OC_DEREFERENCE_POINTER = 1 << 31,
 
-	OC_END       = 0 ///< End of the whole chunk, all 32 bits set to zero
+	OC_END       = 0, ///< End of the whole chunk, all 32 bits set to zero
 };
 
-DECLARE_ENUM_AS_BIT_SET(OldChunkType);
+DECLARE_ENUM_AS_BIT_SET(OldChunkType)
 
 typedef bool OldChunkProc(LoadgameState *ls, int num);
 
