@@ -1,4 +1,4 @@
-/* $Id: screenshot.cpp 18756 2010-01-08 03:17:12Z glx $ */
+/* $Id: screenshot.cpp 26983 2014-10-09 19:52:07Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,7 +10,6 @@
 /** @file screenshot.cpp The creation of screenshots! */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "fileio_func.h"
 #include "viewport_func.h"
 #include "gfx_func.h"
@@ -18,29 +17,55 @@
 #include "blitter/factory.hpp"
 #include "zoom_func.h"
 #include "core/endian_func.hpp"
-#include "map_func.h"
 #include "saveload/saveload.h"
 #include "company_func.h"
 #include "strings_func.h"
-#include "gui.h"
+#include "error.h"
+#include "window_gui.h"
+#include "window_func.h"
+#include "tile_map.h"
+#include "landscape.h"
 
 #include "table/strings.h"
 
+#include "safeguards.h"
 
-char _screenshot_format_name[8];
-uint _num_screenshot_formats;
-uint _cur_screenshot_format;
-static char _screenshot_name[128];
-char _full_screenshot_name[MAX_PATH];
+static const char * const SCREENSHOT_NAME = "screenshot"; ///< Default filename of a saved screenshot.
+static const char * const HEIGHTMAP_NAME  = "heightmap";  ///< Default filename of a saved heightmap.
 
-/* called by the ScreenShot proc to generate screenshot lines. */
+char _screenshot_format_name[8];      ///< Extension of the current screenshot format (corresponds with #_cur_screenshot_format).
+uint _num_screenshot_formats;         ///< Number of available screenshot formats.
+uint _cur_screenshot_format;          ///< Index of the currently selected screenshot format in #_screenshot_formats.
+static char _screenshot_name[128];    ///< Filename of the screenshot file.
+char _full_screenshot_name[MAX_PATH]; ///< Pathname of the screenshot file.
+
+/**
+ * Callback function signature for generating lines of pixel data to be written to the screenshot file.
+ * @param userdata Pointer to user data.
+ * @param buf      Destination buffer.
+ * @param y        Line number of the first line to write.
+ * @param pitch    Number of pixels to write (1 byte for 8bpp, 4 bytes for 32bpp). @see Colour
+ * @param n        Number of lines to write.
+ */
 typedef void ScreenshotCallback(void *userdata, void *buf, uint y, uint pitch, uint n);
+
+/**
+ * Function signature for a screenshot generation routine for one of the available formats.
+ * @param name        Filename, including extension.
+ * @param callb       Callback function for generating lines of pixels.
+ * @param userdata    User data, passed on to \a callb.
+ * @param w           Width of the image in pixels.
+ * @param h           Height of the image in pixels.
+ * @param pixelformat Bits per pixel (bpp), either 8 or 32.
+ * @param palette     %Colour palette (for 8bpp images).
+ * @return File was written successfully.
+ */
 typedef bool ScreenshotHandlerProc(const char *name, ScreenshotCallback *callb, void *userdata, uint w, uint h, int pixelformat, const Colour *palette);
 
+/** Screenshot format information. */
 struct ScreenshotFormat {
-	const char *name;
-	const char *extension;
-	ScreenshotHandlerProc *proc;
+	const char *extension;       ///< File extension.
+	ScreenshotHandlerProc *proc; ///< Function for writing the screenshot.
 };
 
 /*************************************************
@@ -82,12 +107,13 @@ assert_compile(sizeof(RgbQuad) == 4);
  * Generic .BMP writer
  * @param name file name including extension
  * @param callb callback used for gathering rendered image
- * @param userdata parameters forwarded to #callb
+ * @param userdata parameters forwarded to \a callb
  * @param w width in pixels
  * @param h height in pixels
  * @param pixelformat bits per pixel
  * @param palette colour palette (for 8bpp mode)
  * @return was everything ok?
+ * @see ScreenshotHandlerProc
  */
 static bool MakeBMPImage(const char *name, ScreenshotCallback *callb, void *userdata, uint w, uint h, int pixelformat, const Colour *palette)
 {
@@ -204,6 +230,14 @@ static bool MakeBMPImage(const char *name, ScreenshotCallback *callb, void *user
 #if defined(WITH_PNG)
 #include <png.h>
 
+#ifdef PNG_TEXT_SUPPORTED
+#include "rev.h"
+#include "newgrf_config.h"
+#include "ai/ai_info.hpp"
+#include "company_base.h"
+#include "base_media_base.h"
+#endif /* PNG_TEXT_SUPPORTED */
+
 static void PNGAPI png_my_error(png_structp png_ptr, png_const_charp message)
 {
 	DEBUG(misc, 0, "[libpng] error: %s - %s", message, (const char *)png_get_error_ptr(png_ptr));
@@ -215,6 +249,18 @@ static void PNGAPI png_my_warning(png_structp png_ptr, png_const_charp message)
 	DEBUG(misc, 1, "[libpng] warning: %s - %s", message, (const char *)png_get_error_ptr(png_ptr));
 }
 
+/**
+ * Generic .PNG file image writer.
+ * @param name        Filename, including extension.
+ * @param callb       Callback function for generating lines of pixels.
+ * @param userdata    User data, passed on to \a callb.
+ * @param w           Width of the image in pixels.
+ * @param h           Height of the image in pixels.
+ * @param pixelformat Bits per pixel (bpp), either 8 or 32.
+ * @param palette     %Colour palette (for 8bpp images).
+ * @return File was written successfully.
+ * @see ScreenshotHandlerProc
+ */
 static bool MakePNGImage(const char *name, ScreenshotCallback *callb, void *userdata, uint w, uint h, int pixelformat, const Colour *palette)
 {
 	png_color rq[256];
@@ -231,7 +277,7 @@ static bool MakePNGImage(const char *name, ScreenshotCallback *callb, void *user
 	f = fopen(name, "wb");
 	if (f == NULL) return false;
 
-	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, (void *)name, png_my_error, png_my_warning);
+	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, const_cast<char *>(name), png_my_error, png_my_warning);
 
 	if (png_ptr == NULL) {
 		fclose(f);
@@ -258,6 +304,41 @@ static bool MakePNGImage(const char *name, ScreenshotCallback *callb, void *user
 	png_set_IHDR(png_ptr, info_ptr, w, h, 8, pixelformat == 8 ? PNG_COLOR_TYPE_PALETTE : PNG_COLOR_TYPE_RGB,
 		PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 
+#ifdef PNG_TEXT_SUPPORTED
+	/* Try to add some game metadata to the PNG screenshot so
+	 * it's more useful for debugging and archival purposes. */
+	png_text_struct text[2];
+	memset(text, 0, sizeof(text));
+	text[0].key = const_cast<char *>("Software");
+	text[0].text = const_cast<char *>(_openttd_revision);
+	text[0].text_length = strlen(_openttd_revision);
+	text[0].compression = PNG_TEXT_COMPRESSION_NONE;
+
+	char buf[8192];
+	char *p = buf;
+	p += seprintf(p, lastof(buf), "Graphics set: %s (%u)\n", BaseGraphics::GetUsedSet()->name, BaseGraphics::GetUsedSet()->version);
+	p = strecpy(p, "NewGRFs:\n", lastof(buf));
+	for (const GRFConfig *c = _game_mode == GM_MENU ? NULL : _grfconfig; c != NULL; c = c->next) {
+		p += seprintf(p, lastof(buf), "%08X ", BSWAP32(c->ident.grfid));
+		p = md5sumToString(p, lastof(buf), c->ident.md5sum);
+		p += seprintf(p, lastof(buf), " %s\n", c->filename);
+	}
+	p = strecpy(p, "\nCompanies:\n", lastof(buf));
+	const Company *c;
+	FOR_ALL_COMPANIES(c) {
+		if (c->ai_info == NULL) {
+			p += seprintf(p, lastof(buf), "%2i: Human\n", (int)c->index);
+		} else {
+			p += seprintf(p, lastof(buf), "%2i: %s (v%d)\n", (int)c->index, c->ai_info->GetName(), c->ai_info->GetVersion());
+		}
+	}
+	text[1].key = const_cast<char *>("Description");
+	text[1].text = buf;
+	text[1].text_length = p - buf;
+	text[1].compression = PNG_TEXT_COMPRESSION_zTXt;
+	png_set_text(png_ptr, info_ptr, text, 2);
+#endif /* PNG_TEXT_SUPPORTED */
+
 	if (pixelformat == 8) {
 		/* convert the palette to the .PNG format. */
 		for (i = 0; i != 256; i++) {
@@ -307,8 +388,9 @@ static bool MakePNGImage(const char *name, ScreenshotCallback *callb, void *user
 		y += n;
 
 		/* write them to png */
-		for (i = 0; i != n; i++)
+		for (i = 0; i != n; i++) {
 			png_write_row(png_ptr, (png_bytep)buff + i * w * bpp);
+		}
 	} while (y != h);
 
 	png_write_end(png_ptr, info_ptr);
@@ -325,6 +407,7 @@ static bool MakePNGImage(const char *name, ScreenshotCallback *callb, void *user
  **** SCREENSHOT CODE FOR ZSOFT PAINTBRUSH (.PCX)
  *************************************************/
 
+/** Definition of a PCX file header. */
 struct PcxHeader {
 	byte manufacturer;
 	byte version;
@@ -344,6 +427,18 @@ struct PcxHeader {
 };
 assert_compile(sizeof(PcxHeader) == 128);
 
+/**
+ * Generic .PCX file image writer.
+ * @param name        Filename, including extension.
+ * @param callb       Callback function for generating lines of pixels.
+ * @param userdata    User data, passed on to \a callb.
+ * @param w           Width of the image in pixels.
+ * @param h           Height of the image in pixels.
+ * @param pixelformat Bits per pixel (bpp), either 8 or 32.
+ * @param palette     %Colour palette (for 8bpp images).
+ * @return File was written successfully.
+ * @see ScreenshotHandlerProc
+ */
 static bool MakePCXImage(const char *name, ScreenshotCallback *callb, void *userdata, uint w, uint h, int pixelformat, const Colour *palette)
 {
 	FILE *f;
@@ -356,8 +451,7 @@ static bool MakePCXImage(const char *name, ScreenshotCallback *callb, void *user
 		DEBUG(misc, 0, "Can't convert a 32bpp screenshot to PCX format. Please pick another format.");
 		return false;
 	}
-	if (pixelformat != 8 || w == 0)
-		return false;
+	if (pixelformat != 8 || w == 0) return false;
 
 	f = fopen(name, "wb");
 	if (f == NULL) return false;
@@ -413,12 +507,13 @@ static bool MakePCXImage(const char *name, ScreenshotCallback *callb, void *user
 				uint8 ch = bufp[j];
 
 				if (ch != runchar || runcount >= 0x3f) {
-					if (runcount > 1 || (runchar & 0xC0) == 0xC0)
+					if (runcount > 1 || (runchar & 0xC0) == 0xC0) {
 						if (fputc(0xC0 | runcount, f) == EOF) {
 							free(buff);
 							fclose(f);
 							return false;
 						}
+					}
 					if (fputc(runchar, f) == EOF) {
 						free(buff);
 						fclose(f);
@@ -431,12 +526,13 @@ static bool MakePCXImage(const char *name, ScreenshotCallback *callb, void *user
 			}
 
 			/* write remaining bytes.. */
-			if (runcount > 1 || (runchar & 0xC0) == 0xC0)
+			if (runcount > 1 || (runchar & 0xC0) == 0xC0) {
 				if (fputc(0xC0 | runcount, f) == EOF) {
 					free(buff);
 					fclose(f);
 					return false;
 				}
+			}
 			if (fputc(runchar, f) == EOF) {
 				free(buff);
 				fclose(f);
@@ -472,46 +568,48 @@ static bool MakePCXImage(const char *name, ScreenshotCallback *callb, void *user
  **** GENERIC SCREENSHOT CODE
  *************************************************/
 
+/** Available screenshot formats. */
 static const ScreenshotFormat _screenshot_formats[] = {
 #if defined(WITH_PNG)
-	{"PNG", "png", &MakePNGImage},
+	{"png", &MakePNGImage},
 #endif
-	{"BMP", "bmp", &MakeBMPImage},
-	{"PCX", "pcx", &MakePCXImage},
+	{"bmp", &MakeBMPImage},
+	{"pcx", &MakePCXImage},
 };
 
+/** Get filename extension of current screenshot file format. */
+const char *GetCurrentScreenshotExtension()
+{
+	return _screenshot_formats[_cur_screenshot_format].extension;
+}
+
+/** Initialize screenshot format information on startup, with #_screenshot_format_name filled from the loadsave code. */
 void InitializeScreenshotFormats()
 {
-	int i, j;
-	for (i = 0, j = 0; i != lengthof(_screenshot_formats); i++)
+	uint j = 0;
+	for (uint i = 0; i < lengthof(_screenshot_formats); i++) {
 		if (!strcmp(_screenshot_format_name, _screenshot_formats[i].extension)) {
 			j = i;
 			break;
 		}
+	}
 	_cur_screenshot_format = j;
 	_num_screenshot_formats = lengthof(_screenshot_formats);
 }
 
-const char *GetScreenshotFormatDesc(int i)
-{
-	return _screenshot_formats[i].name;
-}
-
-void SetScreenshotFormat(int i)
-{
-	_cur_screenshot_format = i;
-	strecpy(_screenshot_format_name, _screenshot_formats[i].extension, lastof(_screenshot_format_name));
-}
-
-/* screenshot generator that dumps the current video buffer */
+/**
+ * Callback of the screenshot generator that dumps the current video buffer.
+ * @see ScreenshotCallback
+ */
 static void CurrentScreenCallback(void *userdata, void *buf, uint y, uint pitch, uint n)
 {
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 	void *src = blitter->MoveTo(_screen.dst_ptr, 0, y);
 	blitter->CopyImageToBuffer(src, buf, _screen.width, n, pitch);
 }
 
-/** generate a large piece of the world
+/**
+ * generate a large piece of the world
  * @param userdata Viewport area to draw
  * @param buf Videobuffer with same bitdepth as current blitter
  * @param y First line to render
@@ -566,13 +664,20 @@ static void LargeWorldCallback(void *userdata, void *buf, uint y, uint pitch, ui
 	_screen_disable_anim = old_disable_anim;
 }
 
-static const char *MakeScreenshotName(const char *ext)
+/**
+ * Construct a pathname for a screenshot file.
+ * @param default_fn Default filename.
+ * @param ext        Extension to use.
+ * @param crashlog   Create path for crash.png
+ * @return Pathname for a screenshot file.
+ */
+static const char *MakeScreenshotName(const char *default_fn, const char *ext, bool crashlog = false)
 {
 	bool generate = StrEmpty(_screenshot_name);
 
 	if (generate) {
 		if (_game_mode == GM_EDITOR || _game_mode == GM_MENU || _local_company == COMPANY_SPECTATOR) {
-			strecpy(_screenshot_name, "screenshot", lastof(_screenshot_name));
+			strecpy(_screenshot_name, default_fn, lastof(_screenshot_name));
 		} else {
 			GenerateDefaultSaveName(_screenshot_name, lastof(_screenshot_name));
 		}
@@ -580,10 +685,12 @@ static const char *MakeScreenshotName(const char *ext)
 
 	/* Add extension to screenshot file */
 	size_t len = strlen(_screenshot_name);
-	snprintf(&_screenshot_name[len], lengthof(_screenshot_name) - len, ".%s", ext);
+	seprintf(&_screenshot_name[len], lastof(_screenshot_name), ".%s", ext);
+
+	const char *screenshot_dir = crashlog ? _personal_dir : FiosGetScreenshotDir();
 
 	for (uint serial = 1;; serial++) {
-		if (snprintf(_full_screenshot_name, lengthof(_full_screenshot_name), "%s%s", _personal_dir, _screenshot_name) >= (int)lengthof(_full_screenshot_name)) {
+		if (seprintf(_full_screenshot_name, lastof(_full_screenshot_name), "%s%s", screenshot_dir, _screenshot_name) >= (int)lengthof(_full_screenshot_name)) {
 			/* We need more characters than MAX_PATH -> end with error */
 			_full_screenshot_name[0] = '\0';
 			break;
@@ -591,49 +698,130 @@ static const char *MakeScreenshotName(const char *ext)
 		if (!generate) break; // allow overwriting of non-automatic filenames
 		if (!FileExists(_full_screenshot_name)) break;
 		/* If file exists try another one with same name, but just with a higher index */
-		snprintf(&_screenshot_name[len], lengthof(_screenshot_name) - len, "#%u.%s", serial, ext);
+		seprintf(&_screenshot_name[len], lastof(_screenshot_name) - len, "#%u.%s", serial, ext);
 	}
 
 	return _full_screenshot_name;
 }
 
-static bool MakeSmallScreenshot()
+/** Make a screenshot of the current screen. */
+static bool MakeSmallScreenshot(bool crashlog)
 {
 	const ScreenshotFormat *sf = _screenshot_formats + _cur_screenshot_format;
-	return sf->proc(MakeScreenshotName(sf->extension), CurrentScreenCallback, NULL, _screen.width, _screen.height, BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth(), _cur_palette);
+	return sf->proc(MakeScreenshotName(SCREENSHOT_NAME, sf->extension, crashlog), CurrentScreenCallback, NULL, _screen.width, _screen.height,
+			BlitterFactory::GetCurrentBlitter()->GetScreenDepth(), _cur_palette.palette);
 }
 
-static bool MakeWorldScreenshot()
+/**
+ * Configure a ViewPort for rendering (a part of) the map into a screenshot.
+ * @param t Screenshot type
+ * @param [out] vp Result viewport
+ */
+void SetupScreenshotViewport(ScreenshotType t, ViewPort *vp)
+{
+	/* Determine world coordinates of screenshot */
+	if (t == SC_WORLD) {
+		vp->zoom = ZOOM_LVL_WORLD_SCREENSHOT;
+
+		TileIndex north_tile = _settings_game.construction.freeform_edges ? TileXY(1, 1) : TileXY(0, 0);
+		TileIndex south_tile = MapSize() - 1;
+
+		/* We need to account for a hill or high building at tile 0,0. */
+		int extra_height_top = TilePixelHeight(north_tile) + 150;
+		/* If there is a hill at the bottom don't create a large black area. */
+		int reclaim_height_bottom = TilePixelHeight(south_tile);
+
+		vp->virtual_left   = RemapCoords(TileX(south_tile) * TILE_SIZE, TileY(north_tile) * TILE_SIZE, 0).x;
+		vp->virtual_top    = RemapCoords(TileX(north_tile) * TILE_SIZE, TileY(north_tile) * TILE_SIZE, extra_height_top).y;
+		vp->virtual_width  = RemapCoords(TileX(north_tile) * TILE_SIZE, TileY(south_tile) * TILE_SIZE, 0).x                     - vp->virtual_left + 1;
+		vp->virtual_height = RemapCoords(TileX(south_tile) * TILE_SIZE, TileY(south_tile) * TILE_SIZE, reclaim_height_bottom).y - vp->virtual_top  + 1;
+	} else {
+		vp->zoom = (t == SC_ZOOMEDIN) ? _settings_client.gui.zoom_min : ZOOM_LVL_VIEWPORT;
+
+		Window *w = FindWindowById(WC_MAIN_WINDOW, 0);
+		vp->virtual_left   = w->viewport->virtual_left;
+		vp->virtual_top    = w->viewport->virtual_top;
+		vp->virtual_width  = w->viewport->virtual_width;
+		vp->virtual_height = w->viewport->virtual_height;
+	}
+
+	/* Compute pixel coordinates */
+	vp->left = 0;
+	vp->top = 0;
+	vp->width  = UnScaleByZoom(vp->virtual_width,  vp->zoom);
+	vp->height = UnScaleByZoom(vp->virtual_height, vp->zoom);
+	vp->overlay = NULL;
+}
+
+/**
+ * Make a screenshot of the map.
+ * @param t Screenshot type: World or viewport screenshot
+ * @return true on success
+ */
+static bool MakeLargeWorldScreenshot(ScreenshotType t)
 {
 	ViewPort vp;
-	const ScreenshotFormat *sf;
+	SetupScreenshotViewport(t, &vp);
 
-	vp.zoom = ZOOM_LVL_WORLD_SCREENSHOT;
-	vp.left = 0;
-	vp.top = 0;
-	vp.virtual_left = -(int)MapMaxX() * TILE_PIXELS;
-	vp.virtual_top = 0;
-	vp.virtual_width = (MapMaxX() + MapMaxY()) * TILE_PIXELS;
-	vp.width = vp.virtual_width;
-	vp.virtual_height = (MapMaxX() + MapMaxY()) * TILE_PIXELS >> 1;
-	vp.height = vp.virtual_height;
+	const ScreenshotFormat *sf = _screenshot_formats + _cur_screenshot_format;
+	return sf->proc(MakeScreenshotName(SCREENSHOT_NAME, sf->extension), LargeWorldCallback, &vp, vp.width, vp.height,
+			BlitterFactory::GetCurrentBlitter()->GetScreenDepth(), _cur_palette.palette);
+}
 
-	sf = _screenshot_formats + _cur_screenshot_format;
-	return sf->proc(MakeScreenshotName(sf->extension), LargeWorldCallback, &vp, vp.width, vp.height, BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth(), _cur_palette);
+/**
+ * Callback for generating a heightmap. Supports 8bpp grayscale only.
+ * @param userdata Pointer to user data.
+ * @param buf      Destination buffer.
+ * @param y        Line number of the first line to write.
+ * @param pitch    Number of pixels to write (1 byte for 8bpp, 4 bytes for 32bpp). @see Colour
+ * @param n        Number of lines to write.
+ * @see ScreenshotCallback
+ */
+static void HeightmapCallback(void *userdata, void *buffer, uint y, uint pitch, uint n)
+{
+	byte *buf = (byte *)buffer;
+	while (n > 0) {
+		TileIndex ti = TileXY(MapMaxX(), y);
+		for (uint x = MapMaxX(); true; x--) {
+			*buf = 256 * TileHeight(ti) / (1 + _settings_game.construction.max_heightlevel);
+			buf++;
+			if (x == 0) break;
+			ti = TILE_ADDXY(ti, -1, 0);
+		}
+		y++;
+		n--;
+	}
+}
+
+/**
+ * Make a heightmap of the current map.
+ * @param filename Filename to use for saving.
+ */
+bool MakeHeightmapScreenshot(const char *filename)
+{
+	Colour palette[256];
+	for (uint i = 0; i < lengthof(palette); i++) {
+		palette[i].a = 0xff;
+		palette[i].r = i;
+		palette[i].g = i;
+		palette[i].b = i;
+	}
+	const ScreenshotFormat *sf = _screenshot_formats + _cur_screenshot_format;
+	return sf->proc(filename, HeightmapCallback, NULL, MapSizeX(), MapSizeY(), 8, palette);
 }
 
 /**
  * Make an actual screenshot.
  * @param t    the type of screenshot to make.
  * @param name the name to give to the screenshot.
- * @return true iff the screenshow was made succesfully
+ * @return true iff the screenshot was made successfully
  */
 bool MakeScreenshot(ScreenshotType t, const char *name)
 {
 	if (t == SC_VIEWPORT) {
 		/* First draw the dirty parts of the screen and only then change the name
 		 * of the screenshot. This way the screenshot will always show the name
-		 * of the previous screenshot in the 'succesful' message instead of the
+		 * of the previous screenshot in the 'successful' message instead of the
 		 * name of the new screenshot (or an empty name). */
 		UndrawMouseCursor();
 		DrawDirtyBlocks();
@@ -645,23 +833,34 @@ bool MakeScreenshot(ScreenshotType t, const char *name)
 	bool ret;
 	switch (t) {
 		case SC_VIEWPORT:
-		case SC_RAW:
-			ret = MakeSmallScreenshot();
+			ret = MakeSmallScreenshot(false);
 			break;
 
-		case SC_WORLD:
-			ret = MakeWorldScreenshot();
+		case SC_CRASHLOG:
+			ret = MakeSmallScreenshot(true);
 			break;
 
+		case SC_ZOOMEDIN:
+		case SC_DEFAULTZOOM:
+		case SC_WORLD:
+			ret = MakeLargeWorldScreenshot(t);
+			break;
+
+		case SC_HEIGHTMAP: {
+			const ScreenshotFormat *sf = _screenshot_formats + _cur_screenshot_format;
+			ret = MakeHeightmapScreenshot(MakeScreenshotName(HEIGHTMAP_NAME, sf->extension));
+			break;
+		}
+
 		default:
 			NOT_REACHED();
 	}
 
 	if (ret) {
 		SetDParamStr(0, _screenshot_name);
-		ShowErrorMessage(STR_MESSAGE_SCREENSHOT_SUCCESSFULLY, INVALID_STRING_ID, 0, 0);
+		ShowErrorMessage(STR_MESSAGE_SCREENSHOT_SUCCESSFULLY, INVALID_STRING_ID, WL_WARNING);
 	} else {
-		ShowErrorMessage(STR_ERROR_SCREENSHOT_FAILED, INVALID_STRING_ID, 0, 0);
+		ShowErrorMessage(STR_ERROR_SCREENSHOT_FAILED, INVALID_STRING_ID, WL_ERROR);
 	}
 
 	return ret;
