@@ -1,4 +1,4 @@
-/* $Id: squirrel_helper.hpp 23735 2012-01-03 20:26:05Z rubidium $ */
+/* $Id: squirrel_helper.hpp 26774 2014-09-06 17:46:56Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -26,7 +26,7 @@ template <class CL, ScriptType ST> const char *GetClassName();
 namespace SQConvert {
 	/**
 	 * Pointers assigned to this class will be free'd when this instance
-	 *  comes out of scope. Useful to make sure you can use strdup(),
+	 *  comes out of scope. Useful to make sure you can use stredup(),
 	 *  without leaking memory.
 	 */
 	struct SQAutoFreePointers : SmallVector<void *, 1> {
@@ -85,11 +85,11 @@ namespace SQConvert {
 	template <> inline int Return<int8>        (HSQUIRRELVM vm, int8 res)        { sq_pushinteger(vm, res); return 1; }
 	template <> inline int Return<int16>       (HSQUIRRELVM vm, int16 res)       { sq_pushinteger(vm, res); return 1; }
 	template <> inline int Return<int32>       (HSQUIRRELVM vm, int32 res)       { sq_pushinteger(vm, res); return 1; }
-	template <> inline int Return<int64>       (HSQUIRRELVM vm, int64 res)       { sq_pushinteger(vm, ClampToI32(res)); return 1; }
-	template <> inline int Return<Money>       (HSQUIRRELVM vm, Money res)       { sq_pushinteger(vm, ClampToI32(res)); return 1; }
+	template <> inline int Return<int64>       (HSQUIRRELVM vm, int64 res)       { sq_pushinteger(vm, res); return 1; }
+	template <> inline int Return<Money>       (HSQUIRRELVM vm, Money res)       { sq_pushinteger(vm, res); return 1; }
 	template <> inline int Return<bool>        (HSQUIRRELVM vm, bool res)        { sq_pushbool   (vm, res); return 1; }
-	template <> inline int Return<char *>      (HSQUIRRELVM vm, char *res)       { if (res == NULL) sq_pushnull(vm); else { sq_pushstring(vm, OTTD2SQ(res), -1); free(res); } return 1; }
-	template <> inline int Return<const char *>(HSQUIRRELVM vm, const char *res) { if (res == NULL) sq_pushnull(vm); else { sq_pushstring(vm, OTTD2SQ(res), -1); } return 1; }
+	template <> inline int Return<char *>      (HSQUIRRELVM vm, char *res)       { if (res == NULL) sq_pushnull(vm); else { sq_pushstring(vm, res, -1); free(res); } return 1; }
+	template <> inline int Return<const char *>(HSQUIRRELVM vm, const char *res) { if (res == NULL) sq_pushnull(vm); else { sq_pushstring(vm, res, -1); } return 1; }
 	template <> inline int Return<void *>      (HSQUIRRELVM vm, void *res)       { sq_pushuserpointer(vm, res); return 1; }
 	template <> inline int Return<HSQOBJECT>   (HSQUIRRELVM vm, HSQOBJECT res)   { sq_pushobject(vm, res); return 1; }
 
@@ -104,6 +104,8 @@ namespace SQConvert {
 	template <> inline int8        GetParam(ForceType<int8>        , HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr) { SQInteger     tmp; sq_getinteger    (vm, index, &tmp); return tmp; }
 	template <> inline int16       GetParam(ForceType<int16>       , HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr) { SQInteger     tmp; sq_getinteger    (vm, index, &tmp); return tmp; }
 	template <> inline int32       GetParam(ForceType<int32>       , HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr) { SQInteger     tmp; sq_getinteger    (vm, index, &tmp); return tmp; }
+	template <> inline int64       GetParam(ForceType<int64>       , HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr) { SQInteger     tmp; sq_getinteger    (vm, index, &tmp); return tmp; }
+	template <> inline Money       GetParam(ForceType<Money>       , HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr) { SQInteger     tmp; sq_getinteger    (vm, index, &tmp); return tmp; }
 	template <> inline bool        GetParam(ForceType<bool>        , HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr) { SQBool        tmp; sq_getbool       (vm, index, &tmp); return tmp != 0; }
 	template <> inline void       *GetParam(ForceType<void *>      , HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr) { SQUserPointer tmp; sq_getuserpointer(vm, index, &tmp); return tmp; }
 	template <> inline const char *GetParam(ForceType<const char *>, HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr)
@@ -113,7 +115,7 @@ namespace SQConvert {
 
 		const SQChar *tmp;
 		sq_getstring(vm, -1, &tmp);
-		char *tmp_str = strdup(SQ2OTTD(tmp));
+		char *tmp_str = stredup(tmp);
 		sq_poptop(vm);
 		*ptr->Append() = (void *)tmp_str;
 		str_validate(tmp_str, tmp_str + strlen(tmp_str));
@@ -123,7 +125,7 @@ namespace SQConvert {
 	template <> inline Array      *GetParam(ForceType<Array *>,      HSQUIRRELVM vm, int index, SQAutoFreePointers *ptr)
 	{
 		/* Sanity check of the size. */
-		if (sq_getsize(vm, index) > UINT16_MAX) throw sq_throwerror(vm, _SC("an array used as parameter to a function is too large"));
+		if (sq_getsize(vm, index) > UINT16_MAX) throw sq_throwerror(vm, "an array used as parameter to a function is too large");
 
 		SQObject obj;
 		sq_getstackobj(vm, index, &obj);
@@ -138,7 +140,7 @@ namespace SQConvert {
 				*data.Append() = (int32)tmp;
 			} else {
 				sq_pop(vm, 4);
-				throw sq_throwerror(vm, _SC("a member of an array used as parameter to a function is not numeric"));
+				throw sq_throwerror(vm, "a member of an array used as parameter to a function is not numeric");
 			}
 
 			sq_pop(vm, 2);
@@ -747,17 +749,17 @@ namespace SQConvert {
 		/* Protect against calls to a non-static method in a static way */
 		sq_pushroottable(vm);
 		const char *className = GetClassName<Tcls, Ttype>();
-		sq_pushstring(vm, OTTD2SQ(className), -1);
+		sq_pushstring(vm, className, -1);
 		sq_get(vm, -2);
 		sq_pushobject(vm, instance);
-		if (sq_instanceof(vm) != SQTrue) return sq_throwerror(vm, _SC("class method is non-static"));
+		if (sq_instanceof(vm) != SQTrue) return sq_throwerror(vm, "class method is non-static");
 		sq_pop(vm, 3);
 
 		/* Get the 'real' instance of this class */
 		sq_getinstanceup(vm, 1, &real_instance, 0);
 		/* Get the real function pointer */
 		sq_getuserdata(vm, nparam, &ptr, 0);
-		if (real_instance == NULL) return sq_throwerror(vm, _SC("couldn't detect real instance of class for non-static call"));
+		if (real_instance == NULL) return sq_throwerror(vm, "couldn't detect real instance of class for non-static call");
 		/* Remove the userdata from the stack */
 		sq_pop(vm, 1);
 
@@ -789,17 +791,17 @@ namespace SQConvert {
 		/* Protect against calls to a non-static method in a static way */
 		sq_pushroottable(vm);
 		const char *className = GetClassName<Tcls, Ttype>();
-		sq_pushstring(vm, OTTD2SQ(className), -1);
+		sq_pushstring(vm, className, -1);
 		sq_get(vm, -2);
 		sq_pushobject(vm, instance);
-		if (sq_instanceof(vm) != SQTrue) return sq_throwerror(vm, _SC("class method is non-static"));
+		if (sq_instanceof(vm) != SQTrue) return sq_throwerror(vm, "class method is non-static");
 		sq_pop(vm, 3);
 
 		/* Get the 'real' instance of this class */
 		sq_getinstanceup(vm, 1, &real_instance, 0);
 		/* Get the real function pointer */
 		sq_getuserdata(vm, nparam, &ptr, 0);
-		if (real_instance == NULL) return sq_throwerror(vm, _SC("couldn't detect real instance of class for non-static call"));
+		if (real_instance == NULL) return sq_throwerror(vm, "couldn't detect real instance of class for non-static call");
 		/* Remove the userdata from the stack */
 		sq_pop(vm, 1);
 
