@@ -1,4 +1,4 @@
-/* $Id: squirrel_std.cpp 18862 2010-01-18 15:41:38Z rubidium $ */
+/* $Id: squirrel_std.cpp 26782 2014-09-06 20:22:59Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -9,14 +9,16 @@
 
 /** @file squirrel_std.cpp Implements the Squirrel Standard Function class */
 
+#include "../stdafx.h"
 #include <squirrel.h>
 #include <sqstdmath.h>
-#include "../stdafx.h"
 #include "../debug.h"
-#include "squirrel.hpp"
 #include "squirrel_std.hpp"
 #include "../core/alloc_func.hpp"
 #include "../core/math_func.hpp"
+#include "../string_func.h"
+
+#include "../safeguards.h"
 
 
 SQInteger SquirrelStd::min(HSQUIRRELVM vm)
@@ -43,7 +45,6 @@ SQInteger SquirrelStd::require(HSQUIRRELVM vm)
 {
 	SQInteger top = sq_gettop(vm);
 	const SQChar *filename;
-	SQChar *real_filename;
 
 	sq_getstring(vm, 2, &filename);
 
@@ -54,30 +55,26 @@ SQInteger SquirrelStd::require(HSQUIRRELVM vm)
 		DEBUG(misc, 0, "[squirrel] Couldn't detect the script-name of the 'require'-caller; this should never happen!");
 		return SQ_ERROR;
 	}
-	real_filename = scstrdup(si.source);
+
+	char path[MAX_PATH];
+	strecpy(path, si.source, lastof(path));
 	/* Keep the dir, remove the rest */
-	SQChar *s = scstrrchr(real_filename, PATHSEPCHAR);
+	SQChar *s = strrchr(path, PATHSEPCHAR);
 	if (s != NULL) {
 		/* Keep the PATHSEPCHAR there, remove the rest */
 		s++;
 		*s = '\0';
 	}
-	/* And now we concat, so we are relative from the current script
-	 * First, we have to make sure we have enough space for the full path */
-	real_filename = ReallocT(real_filename, scstrlen(real_filename) + scstrlen(filename) + 1);
-	scstrcat(real_filename, filename);
-	/* Tars dislike opening files with '/' on Windows.. so convert it to '\\' ;) */
-	char *filen = strdup(SQ2OTTD(real_filename));
+	strecat(path, filename, lastof(path));
 #if (PATHSEPCHAR != '/')
-	for (char *n = filen; *n != '\0'; n++) if (*n == '/') *n = PATHSEPCHAR;
+	for (char *n = path; *n != '\0'; n++) if (*n == '/') *n = PATHSEPCHAR;
 #endif
 
-	bool ret = Squirrel::LoadScript(vm, filen);
+	Squirrel *engine = (Squirrel *)sq_getforeignptr(vm);
+	bool ret = engine->LoadScript(vm, path);
 
 	/* Reset the top, so the stack stays correct */
 	sq_settop(vm, top);
-	free(real_filename);
-	free(filen);
 
 	return ret ? 0 : SQ_ERROR;
 }
