@@ -1,4 +1,4 @@
-/* $Id: signs_base.h 18809 2010-01-15 16:41:15Z rubidium $ */
+/* $Id: signs_base.h 23104 2011-11-04 11:09:06Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -25,15 +25,10 @@ struct Sign : SignPool::PoolItem<&_sign_pool> {
 	ViewportSign sign;
 	int32        x;
 	int32        y;
-	byte         z;
+	int32        z;
 	OwnerByte    owner; // placed by this company. Anyone can delete them though. OWNER_NONE for gray signs from old games.
 
-	/**
-	 * Creates a new sign
-	 */
 	Sign(Owner owner = INVALID_OWNER);
-
-	/** Destroy the sign */
 	~Sign();
 
 	void UpdateVirtCoord();
