@@ -1,4 +1,4 @@
-/* $Id: slope_type.h 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: slope_type.h 23595 2011-12-19 17:48:04Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -79,6 +79,15 @@ enum Slope {
 };
 DECLARE_ENUM_AS_BIT_SET(Slope)
 
+/**
+ * Helper for creating a bitset of slopes.
+ * @param x The slope to convert into a bitset.
+ */
+#define M(x) (1 << (x))
+/** Constant bitset with safe slopes for building a level crossing. */
+static const uint32 VALID_LEVEL_CROSSING_SLOPES = M(SLOPE_SEN) | M(SLOPE_ENW) | M(SLOPE_NWS) | M(SLOPE_NS) | M(SLOPE_WSE) | M(SLOPE_EW) | M(SLOPE_FLAT);
+#undef M
+
 
 /**
  * Enumeration for Foundations.
@@ -103,7 +112,7 @@ enum Foundation {
 	FOUNDATION_RAIL_E,           ///< Foundation for TRACK_BIT_RIGHT, but not a leveled foundation.
 	FOUNDATION_RAIL_N,           ///< Foundation for TRACK_BIT_UPPER, but not a leveled foundation.
 
-	FOUNDATION_INVALID = 0xFF    ///< Used inside "rail_cmd.cpp" to indicate invalid slope/track combination.
+	FOUNDATION_INVALID = 0xFF,   ///< Used inside "rail_cmd.cpp" to indicate invalid slope/track combination.
 };
 
 #endif /* SLOPE_TYPE_H */
