@@ -1,4 +1,4 @@
-/* $Id: win32_s.h 22410 2011-05-02 16:14:23Z rubidium $ */
+/* $Id: win32_s.h 26108 2013-11-25 14:30:22Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,7 +15,7 @@
 #include "sound_driver.hpp"
 
 /** Implementation of the sound driver for Windows. */
-class SoundDriver_Win32: public SoundDriver {
+class SoundDriver_Win32 : public SoundDriver {
 public:
 	/* virtual */ const char *Start(const char * const *param);
 
@@ -24,12 +24,10 @@ public:
 };
 
 /** Factory for the sound driver for Windows. */
-class FSoundDriver_Win32: public SoundDriverFactory<FSoundDriver_Win32> {
+class FSoundDriver_Win32 : public DriverFactoryBase {
 public:
-	static const int priority = 10;
-	/* virtual */ const char *GetName() { return "win32"; }
-	/* virtual */ const char *GetDescription() { return "Win32 WaveOut Driver"; }
-	/* virtual */ Driver *CreateInstance() { return new SoundDriver_Win32(); }
+	FSoundDriver_Win32() : DriverFactoryBase(Driver::DT_SOUND, 10, "win32", "Win32 WaveOut Sound Driver") {}
+	/* virtual */ Driver *CreateInstance() const { return new SoundDriver_Win32(); }
 };
 
 #endif /* SOUND_WIN32_H */
