@@ -1,4 +1,4 @@
-/* $Id: strgen.cpp 26544 2014-04-29 18:41:19Z frosch $ */
+/* $Id: strgen.cpp 26521 2014-04-26 20:55:08Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -39,6 +39,8 @@
 
 #include "../table/strgen_tables.h"
 
+#include "../safeguards.h"
+
 
 #ifdef _MSC_VER
 # define LINE_NUM_FMT(s) "%s (%d): warning: %s (" s ")\n"
@@ -51,7 +53,7 @@ void CDECL strgen_warning(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("warning"), _file, _cur_line, buf);
 	_warnings++;
@@ -62,7 +64,7 @@ void CDECL strgen_error(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("error"), _file, _cur_line, buf);
 	_errors++;
@@ -73,7 +75,7 @@ void NORETURN CDECL strgen_fatal(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("FATAL"), _file, _cur_line, buf);
 #ifdef _MSC_VER
@@ -87,7 +89,7 @@ void NORETURN CDECL error(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("FATAL"), _file, _cur_line, buf);
 #ifdef _MSC_VER
@@ -120,9 +122,9 @@ struct FileStringReader : StringReader {
 		fclose(this->fh);
 	}
 
-	/* virtual */ char *ReadLine(char *buffer, size_t size)
+	/* virtual */ char *ReadLine(char *buffer, const char *last)
 	{
-		return fgets(buffer, ClampToU16(size), this->fh);
+		return fgets(buffer, ClampToU16(last - buffer + 1), this->fh);
 	}
 
 	/* virtual */ void HandlePragma(char *str);
@@ -245,7 +247,7 @@ struct FileWriter {
 	 */
 	FileWriter(const char *filename)
 	{
-		this->filename = strdup(filename);
+		this->filename = stredup(filename);
 		this->fh = fopen(this->filename, "wb");
 
 		if (this->fh == NULL) {
@@ -283,7 +285,7 @@ struct HeaderFileWriter : HeaderWriter, FileWriter {
 	 * @param filename The file to open.
 	 */
 	HeaderFileWriter(const char *filename) : FileWriter("tmp.xxx"),
-		real_filename(strdup(filename)), prev(0)
+		real_filename(stredup(filename)), prev(0)
 	{
 		fprintf(this->fh, "/* This file is automatically generated. Do not modify */\n\n");
 		fprintf(this->fh, "#ifndef TABLE_STRINGS_H\n");
@@ -384,13 +386,13 @@ static inline void ottd_mkdir(const char *directory)
  * path separator and the filename. The separator is only appended if the path
  * does not already end with a separator
  */
-static inline char *mkpath(char *buf, size_t buflen, const char *path, const char *file)
+static inline char *mkpath(char *buf, const char *last, const char *path, const char *file)
 {
-	ttd_strlcpy(buf, path, buflen); // copy directory into buffer
+	strecpy(buf, path, last); // copy directory into buffer
 
 	char *p = strchr(buf, '\0'); // add path separator if necessary
-	if (p[-1] != PATHSEPCHAR && (size_t)(p - buf) + 1 < buflen) *p++ = PATHSEPCHAR;
-	ttd_strlcpy(p, file, buflen - (size_t)(p - buf)); // concatenate filename at end of buffer
+	if (p[-1] != PATHSEPCHAR && p != last) *p++ = PATHSEPCHAR;
+	strecpy(p, file, last); // concatenate filename at end of buffer
 	return buf;
 }
 
@@ -437,7 +439,7 @@ int CDECL main(int argc, char *argv[])
 
 		switch (i) {
 			case 'v':
-				puts("$Revision: 26544 $");
+				puts("$Revision: 26521 $");
 				return 0;
 
 			case 'C':
@@ -482,7 +484,7 @@ int CDECL main(int argc, char *argv[])
 
 			case 'h':
 				puts(
-					"strgen - $Revision: 26544 $\n"
+					"strgen - $Revision: 26521 $\n"
 					" -v | --version    print version information and exit\n"
 					" -t | --todo       replace any untranslated strings with '<TODO>'\n"
 					" -w | --warning    print a warning for any untranslated strings\n"
@@ -520,7 +522,7 @@ int CDECL main(int argc, char *argv[])
 		 * with a (free) parameter the program will translate that language to destination
 		 * directory. As input english.txt is parsed from the source directory */
 		if (mgo.numleft == 0) {
-			mkpath(pathbuf, lengthof(pathbuf), src_dir, "english.txt");
+			mkpath(pathbuf, lastof(pathbuf), src_dir, "english.txt");
 
 			/* parse master file */
 			StringData data(TAB_COUNT);
@@ -530,7 +532,7 @@ int CDECL main(int argc, char *argv[])
 
 			/* write strings.h */
 			ottd_mkdir(dest_dir);
-			mkpath(pathbuf, lengthof(pathbuf), dest_dir, "strings.h");
+			mkpath(pathbuf, lastof(pathbuf), dest_dir, "strings.h");
 
 			HeaderFileWriter writer(pathbuf);
 			writer.WriteHeader(data);
@@ -538,7 +540,7 @@ int CDECL main(int argc, char *argv[])
 		} else if (mgo.numleft >= 1) {
 			char *r;
 
-			mkpath(pathbuf, lengthof(pathbuf), src_dir, "english.txt");
+			mkpath(pathbuf, lastof(pathbuf), src_dir, "english.txt");
 
 			StringData data(TAB_COUNT);
 			/* parse master file and check if target file is correct */
@@ -556,7 +558,7 @@ int CDECL main(int argc, char *argv[])
 
 				/* get the targetfile, strip any directories and append to destination path */
 				r = strrchr(mgo.argv[i], PATHSEPCHAR);
-				mkpath(pathbuf, lengthof(pathbuf), dest_dir, (r != NULL) ? &r[1] : mgo.argv[i]);
+				mkpath(pathbuf, lastof(pathbuf), dest_dir, (r != NULL) ? &r[1] : mgo.argv[i]);
 
 				/* rename the .txt (input-extension) to .lng */
 				r = strrchr(pathbuf, '.');
