@@ -1,4 +1,4 @@
-/* $Id: timetable_cmd.cpp 25617 2013-07-17 18:37:13Z rubidium $ */
+/* $Id: timetable_cmd.cpp 27348 2015-07-30 18:45:29Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -20,51 +20,62 @@
 
 #include "table/strings.h"
 
+#include "safeguards.h"
+
 /**
  * Change/update a particular timetable entry.
  * @param v            The vehicle to change the timetable of.
  * @param order_number The index of the timetable in the order list.
  * @param val          The new data of the timetable entry.
  * @param mtf          Which part of the timetable entry to change.
+ * @param timetabled   If the new value is explicitly timetabled.
  */
-static void ChangeTimetable(Vehicle *v, VehicleOrderID order_number, uint16 val, ModifyTimetableFlags mtf)
+static void ChangeTimetable(Vehicle *v, VehicleOrderID order_number, uint16 val, ModifyTimetableFlags mtf, bool timetabled)
 {
 	Order *order = v->GetOrder(order_number);
-	int delta = 0;
+	int total_delta = 0;
+	int timetable_delta = 0;
 
 	switch (mtf) {
 		case MTF_WAIT_TIME:
-			delta = val - order->wait_time;
-			order->wait_time = val;
+			total_delta = val - order->GetWaitTime();
+			timetable_delta = (timetabled ? val : 0) - order->GetTimetabledWait();
+			order->SetWaitTime(val);
+			order->SetWaitTimetabled(timetabled);
 			break;
 
 		case MTF_TRAVEL_TIME:
-			delta = val - order->travel_time;
-			order->travel_time = val;
+			total_delta = val - order->GetTravelTime();
+			timetable_delta = (timetabled ? val : 0) - order->GetTimetabledTravel();
+			order->SetTravelTime(val);
+			order->SetTravelTimetabled(timetabled);
 			break;
 
 		case MTF_TRAVEL_SPEED:
-			order->max_speed = val;
+			order->SetMaxSpeed(val);
 			break;
 
 		default:
 			NOT_REACHED();
 	}
-	v->orders.list->UpdateOrderTimetable(delta);
+	v->orders.list->UpdateTotalDuration(total_delta);
+	v->orders.list->UpdateTimetableDuration(timetable_delta);
 
 	for (v = v->FirstShared(); v != NULL; v = v->NextShared()) {
 		if (v->cur_real_order_index == order_number && v->current_order.Equals(*order)) {
 			switch (mtf) {
 				case MTF_WAIT_TIME:
-					v->current_order.wait_time = val;
+					v->current_order.SetWaitTime(val);
+					v->current_order.SetWaitTimetabled(timetabled);
 					break;
 
 				case MTF_TRAVEL_TIME:
-					v->current_order.travel_time = val;
+					v->current_order.SetTravelTime(val);
+					v->current_order.SetTravelTimetabled(timetabled);
 					break;
 
 				case MTF_TRAVEL_SPEED:
-					v->current_order.max_speed = val;
+					v->current_order.SetMaxSpeed(val);
 					break;
 
 				default:
@@ -85,6 +96,7 @@ static void ChangeTimetable(Vehicle *v, VehicleOrderID order_number, uint16 val,
  * - p1 = (bit 28-29) - Timetable data to change (@see ModifyTimetableFlags)
  * @param p2 The amount of time to wait.
  * - p2 = (bit  0-15) - The data to modify as specified by p1 bits 28-29.
+ *                      0 to clear times, UINT16_MAX to clear speed limit.
  * @param text unused
  * @return the cost of this operation or an error
  */
@@ -105,9 +117,9 @@ CommandCost CmdChangeTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 	ModifyTimetableFlags mtf = Extract<ModifyTimetableFlags, 28, 2>(p1);
 	if (mtf >= MTF_END) return CMD_ERROR;
 
-	int wait_time   = order->wait_time;
-	int travel_time = order->travel_time;
-	int max_speed   = order->max_speed;
+	int wait_time   = order->GetWaitTime();
+	int travel_time = order->GetTravelTime();
+	int max_speed   = order->GetMaxSpeed();
 	switch (mtf) {
 		case MTF_WAIT_TIME:
 			wait_time = GB(p2, 0, 16);
@@ -126,7 +138,7 @@ CommandCost CmdChangeTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 			NOT_REACHED();
 	}
 
-	if (wait_time != order->wait_time) {
+	if (wait_time != order->GetWaitTime()) {
 		switch (order->GetType()) {
 			case OT_GOTO_STATION:
 				if (order->GetNonStopType() & ONSF_NO_STOP_AT_DESTINATION_STATION) return_cmd_error(STR_ERROR_TIMETABLE_NOT_STOPPING_HERE);
@@ -139,13 +151,34 @@ CommandCost CmdChangeTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 		}
 	}
 
-	if (travel_time != order->travel_time && order->IsType(OT_CONDITIONAL)) return CMD_ERROR;
-	if (max_speed != order->max_speed && (order->IsType(OT_CONDITIONAL) || v->type == VEH_AIRCRAFT)) return CMD_ERROR;
+	if (travel_time != order->GetTravelTime() && order->IsType(OT_CONDITIONAL)) return CMD_ERROR;
+	if (max_speed != order->GetMaxSpeed() && (order->IsType(OT_CONDITIONAL) || v->type == VEH_AIRCRAFT)) return CMD_ERROR;
 
 	if (flags & DC_EXEC) {
-		if (wait_time   != order->wait_time)   ChangeTimetable(v, order_number, wait_time,   MTF_WAIT_TIME);
-		if (travel_time != order->travel_time) ChangeTimetable(v, order_number, travel_time, MTF_TRAVEL_TIME);
-		if (max_speed   != order->max_speed)   ChangeTimetable(v, order_number, max_speed,   MTF_TRAVEL_SPEED);
+		switch (mtf) {
+			case MTF_WAIT_TIME:
+				/* Set time if changing the value or confirming an estimated time as timetabled. */
+				if (wait_time != order->GetWaitTime() || (wait_time > 0 && !order->IsWaitTimetabled())) {
+					ChangeTimetable(v, order_number, wait_time, MTF_WAIT_TIME, wait_time > 0);
+				}
+				break;
+
+			case MTF_TRAVEL_TIME:
+				/* Set time if changing the value or confirming an estimated time as timetabled. */
+				if (travel_time != order->GetTravelTime() || (travel_time > 0 && !order->IsTravelTimetabled())) {
+					ChangeTimetable(v, order_number, travel_time, MTF_TRAVEL_TIME, travel_time > 0);
+				}
+				break;
+
+			case MTF_TRAVEL_SPEED:
+				if (max_speed != order->GetMaxSpeed()) {
+					ChangeTimetable(v, order_number, max_speed, MTF_TRAVEL_SPEED, max_speed != UINT16_MAX);
+				}
+				break;
+
+			default:
+				break;
+		}
 	}
 
 	return CommandCost();
@@ -345,13 +378,15 @@ CommandCost CmdAutofillTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1,
  */
 void UpdateVehicleTimetable(Vehicle *v, bool travelling)
 {
-	uint timetabled = travelling ? v->current_order.travel_time : v->current_order.wait_time;
 	uint time_taken = v->current_order_time;
 
 	v->current_order_time = 0;
 
 	if (v->current_order.IsType(OT_IMPLICIT)) return; // no timetabling of auto orders
 
+	if (v->cur_real_order_index >= v->GetNumOrders()) return;
+	Order *real_current_order = v->GetOrder(v->cur_real_order_index);
+
 	VehicleOrderID first_manual_order = 0;
 	for (Order *o = v->GetFirstOrder(); o != NULL && o->IsType(OT_IMPLICIT); o = o->next) {
 		++first_manual_order;
@@ -378,43 +413,52 @@ void UpdateVehicleTimetable(Vehicle *v, bool travelling)
 
 	if (!HasBit(v->vehicle_flags, VF_TIMETABLE_STARTED)) return;
 
-	if (HasBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE)) {
-		if (travelling && !HasBit(v->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME)) {
-			/* Need to clear that now as otherwise we are not able to reduce the wait time */
-			v->current_order.wait_time = 0;
-		}
+	bool autofilling = HasBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE);
+	bool remeasure_wait_time = !real_current_order->IsWaitTimetabled() ||
+			(autofilling && !HasBit(v->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME));
 
-		if (just_started) return;
-
-		/* Modify station waiting time only if our new value is larger (this is
-		 * always the case when we cleared the timetable). */
-		if (!v->current_order.IsType(OT_CONDITIONAL) && (travelling || time_taken > v->current_order.wait_time)) {
-			/* Round the time taken up to the nearest day, as this will avoid
-			 * confusion for people who are timetabling in days, and can be
-			 * adjusted later by people who aren't.
-			 * For trains/aircraft multiple movement cycles are done in one
-			 * tick. This makes it possible to leave the station and process
-			 * e.g. a depot order in the same tick, causing it to not fill
-			 * the timetable entry like is done for road vehicles/ships.
-			 * Thus always make sure at least one tick is used between the
-			 * processing of different orders when filling the timetable. */
-			time_taken = CeilDiv(max(time_taken, 1U), DAY_TICKS) * DAY_TICKS;
-
-			ChangeTimetable(v, v->cur_real_order_index, time_taken, travelling ? MTF_TRAVEL_TIME : MTF_WAIT_TIME);
-		}
-
-		if (v->cur_real_order_index == first_manual_order && travelling) {
-			/* If we just started we would have returned earlier and have not reached
-			 * this code. So obviously, we have completed our round: So turn autofill
-			 * off again. */
-			ClrBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE);
-			ClrBit(v->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME);
-		}
-		return;
+	if (travelling && remeasure_wait_time) {
+		/* We just finished travelling and want to remeasure the loading time,
+		 * so do not apply any restrictions for the loading to finish. */
+		v->current_order.SetWaitTime(0);
 	}
 
 	if (just_started) return;
 
+	/* Before modifying waiting times, check whether we want to preserve bigger ones. */
+	if (!real_current_order->IsType(OT_CONDITIONAL) &&
+			(travelling || time_taken > real_current_order->GetWaitTime() || remeasure_wait_time)) {
+		/* Round the time taken up to the nearest day, as this will avoid
+		 * confusion for people who are timetabling in days, and can be
+		 * adjusted later by people who aren't.
+		 * For trains/aircraft multiple movement cycles are done in one
+		 * tick. This makes it possible to leave the station and process
+		 * e.g. a depot order in the same tick, causing it to not fill
+		 * the timetable entry like is done for road vehicles/ships.
+		 * Thus always make sure at least one tick is used between the
+		 * processing of different orders when filling the timetable. */
+		uint time_to_set = CeilDiv(max(time_taken, 1U), DAY_TICKS) * DAY_TICKS;
+
+		if (travelling && (autofilling || !real_current_order->IsTravelTimetabled())) {
+			ChangeTimetable(v, v->cur_real_order_index, time_to_set, MTF_TRAVEL_TIME, autofilling);
+		} else if (!travelling && (autofilling || !real_current_order->IsWaitTimetabled())) {
+			ChangeTimetable(v, v->cur_real_order_index, time_to_set, MTF_WAIT_TIME, autofilling);
+		}
+	}
+
+	if (v->cur_real_order_index == first_manual_order && travelling) {
+		/* If we just started we would have returned earlier and have not reached
+		 * this code. So obviously, we have completed our round: So turn autofill
+		 * off again. */
+		ClrBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE);
+		ClrBit(v->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME);
+	}
+
+	if (autofilling) return;
+
+	uint timetabled = travelling ? real_current_order->GetTimetabledTravel() :
+			real_current_order->GetTimetabledWait();
+
 	/* Vehicles will wait at stations if they arrive early even if they are not
 	 * timetabled to wait there, so make sure the lateness counter is updated
 	 * when this happens. */
