@@ -1,4 +1,4 @@
-/* $Id: viewport_type.h 24900 2013-01-08 22:46:42Z planetmaker $ */
+/* $Id: viewport_type.h 27271 2015-05-08 17:30:07Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,6 +14,9 @@
 
 #include "zoom_type.h"
 #include "strings_type.h"
+#include "table/strings.h"
+
+class LinkGraphOverlay;
 
 /**
  * Data structure for viewport, display of a part of the world
@@ -30,6 +33,7 @@ struct ViewPort {
 	int virtual_height;  ///< height << zoom
 
 	ZoomLevel zoom; ///< The zoom level of the viewport.
+	LinkGraphOverlay *overlay;
 };
 
 /** Margins for the viewport sign */
@@ -47,7 +51,7 @@ struct ViewportSign {
 	uint16 width_normal; ///< The width when not zoomed out (normal font)
 	uint16 width_small;  ///< The width when zoomed out (small font)
 
-	void UpdatePosition(int center, int top, StringID str);
+	void UpdatePosition(int center, int top, StringID str, StringID str_small = STR_NULL);
 	void MarkDirty(ZoomLevel maxzoom = ZOOM_LVL_MAX) const;
 };
 
