@@ -1,4 +1,4 @@
-/* $Id: water_cmd.cpp 23403 2011-12-03 21:31:44Z rubidium $ */
+/* $Id: water_cmd.cpp 27272 2015-05-08 17:32:57Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -32,12 +32,18 @@
 #include "tunnelbridge_map.h"
 #include "station_base.h"
 #include "ai/ai.hpp"
+#include "game/game.hpp"
 #include "core/random_func.hpp"
 #include "core/backup_type.hpp"
 #include "date_func.h"
+#include "company_base.h"
+#include "company_gui.h"
+#include "newgrf_generic.h"
 
 #include "table/strings.h"
 
+#include "safeguards.h"
+
 /**
  * Describes from which directions a specific slope can be flooded (if the tile is floodable at all).
  */
@@ -103,10 +109,9 @@ CommandCost CmdBuildShipDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, ui
 		return_cmd_error(STR_ERROR_MUST_BE_BUILT_ON_WATER);
 	}
 
-	if ((MayHaveBridgeAbove(tile) && IsBridgeAbove(tile)) ||
-		(MayHaveBridgeAbove(tile2) && IsBridgeAbove(tile2))) return_cmd_error(STR_ERROR_MUST_DEMOLISH_BRIDGE_FIRST);
+	if (IsBridgeAbove(tile) || IsBridgeAbove(tile2)) return_cmd_error(STR_ERROR_MUST_DEMOLISH_BRIDGE_FIRST);
 
-	if (GetTileSlope(tile, NULL) != SLOPE_FLAT || GetTileSlope(tile2, NULL) != SLOPE_FLAT) {
+	if (!IsTileFlat(tile) || !IsTileFlat(tile2)) {
 		/* Prevent depots on rapids */
 		return_cmd_error(STR_ERROR_SITE_UNSUITABLE);
 	}
@@ -134,8 +139,15 @@ CommandCost CmdBuildShipDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, ui
 		Depot *depot = new Depot(tile);
 		depot->build_date = _date;
 
-		MakeShipDepot(tile,  _current_company, depot->index, DEPOT_NORTH, axis, wc1);
-		MakeShipDepot(tile2, _current_company, depot->index, DEPOT_SOUTH, axis, wc2);
+		if (wc1 == WATER_CLASS_CANAL || wc2 == WATER_CLASS_CANAL) {
+			/* Update infrastructure counts after the unconditional clear earlier. */
+			Company::Get(_current_company)->infrastructure.water += wc1 == WATER_CLASS_CANAL && wc2 == WATER_CLASS_CANAL ? 2 : 1;
+		}
+		Company::Get(_current_company)->infrastructure.water += 2 * LOCK_DEPOT_TILE_FACTOR;
+		DirtyCompanyInfrastructureWindows(_current_company);
+
+		MakeShipDepot(tile,  _current_company, depot->index, DEPOT_PART_NORTH, axis, wc1);
+		MakeShipDepot(tile2, _current_company, depot->index, DEPOT_PART_SOUTH, axis, wc2);
 		MarkTileDirtyByTile(tile);
 		MarkTileDirtyByTile(tile2);
 		MakeDefaultName(depot);
@@ -149,10 +161,37 @@ void MakeWaterKeepingClass(TileIndex tile, Owner o)
 	WaterClass wc = GetWaterClass(tile);
 
 	/* Autoslope might turn an originally canal or river tile into land */
-	uint z;
-	if (GetTileSlope(tile, &z) != SLOPE_FLAT) wc = WATER_CLASS_INVALID;
+	int z;
+	Slope slope = GetTileSlope(tile, &z);
 
-	if (wc == WATER_CLASS_SEA && z > 0) wc = WATER_CLASS_CANAL;
+	if (slope != SLOPE_FLAT) {
+		if (wc == WATER_CLASS_CANAL) {
+			/* If we clear the canal, we have to remove it from the infrastructure count as well. */
+			Company *c = Company::GetIfValid(o);
+			if (c != NULL) {
+				c->infrastructure.water--;
+				DirtyCompanyInfrastructureWindows(c->index);
+			}
+			/* Sloped canals are locks and no natural water remains whatever the slope direction */
+			wc = WATER_CLASS_INVALID;
+		}
+
+		/* Only river water should be restored on appropriate slopes. Other water would be invalid on slopes */
+		if (wc != WATER_CLASS_RIVER || GetInclinedSlopeDirection(slope) == INVALID_DIAGDIR) {
+			wc = WATER_CLASS_INVALID;
+		}
+	}
+
+	if (wc == WATER_CLASS_SEA && z > 0) {
+		/* Update company infrastructure count. */
+		Company *c = Company::GetIfValid(o);
+		if (c != NULL) {
+			c->infrastructure.water++;
+			DirtyCompanyInfrastructureWindows(c->index);
+		}
+
+		wc = WATER_CLASS_CANAL;
+	}
 
 	/* Zero map array and terminate animation */
 	DoClearSquare(tile);
@@ -187,6 +226,12 @@ static CommandCost RemoveShipDepot(TileIndex tile, DoCommandFlag flags)
 	if (flags & DC_EXEC) {
 		delete Depot::GetByTile(tile);
 
+		Company *c = Company::GetIfValid(GetTileOwner(tile));
+		if (c != NULL) {
+			c->infrastructure.water -= 2 * LOCK_DEPOT_TILE_FACTOR;
+			DirtyCompanyInfrastructureWindows(c->index);
+		}
+
 		MakeWaterKeepingClass(tile,  GetTileOwner(tile));
 		MakeWaterKeepingClass(tile2, GetTileOwner(tile2));
 	}
@@ -212,44 +257,52 @@ static CommandCost DoBuildLock(TileIndex tile, DiagDirection dir, DoCommandFlag
 	if (ret.Failed()) return ret;
 
 	/* middle tile */
+	WaterClass wc_middle = IsWaterTile(tile) ? GetWaterClass(tile) : WATER_CLASS_CANAL;
 	ret = DoCommand(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 	if (ret.Failed()) return ret;
 	cost.AddCost(ret);
 
 	/* lower tile */
-	WaterClass wc_lower = IsWaterTile(tile - delta) ? GetWaterClass(tile - delta) : WATER_CLASS_CANAL;
-
 	if (!IsWaterTile(tile - delta)) {
 		ret = DoCommand(tile - delta, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 		if (ret.Failed()) return ret;
 		cost.AddCost(ret);
 		cost.AddCost(_price[PR_BUILD_CANAL]);
 	}
-	if (GetTileSlope(tile - delta, NULL) != SLOPE_FLAT) {
+	if (!IsTileFlat(tile - delta)) {
 		return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
 	}
+	WaterClass wc_lower = IsWaterTile(tile - delta) ? GetWaterClass(tile - delta) : WATER_CLASS_CANAL;
 
 	/* upper tile */
-	WaterClass wc_upper = IsWaterTile(tile + delta) ? GetWaterClass(tile + delta) : WATER_CLASS_CANAL;
-
 	if (!IsWaterTile(tile + delta)) {
 		ret = DoCommand(tile + delta, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 		if (ret.Failed()) return ret;
 		cost.AddCost(ret);
 		cost.AddCost(_price[PR_BUILD_CANAL]);
 	}
-	if (GetTileSlope(tile + delta, NULL) != SLOPE_FLAT) {
+	if (!IsTileFlat(tile + delta)) {
 		return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
 	}
+	WaterClass wc_upper = IsWaterTile(tile + delta) ? GetWaterClass(tile + delta) : WATER_CLASS_CANAL;
 
-	if ((MayHaveBridgeAbove(tile) && IsBridgeAbove(tile)) ||
-	    (MayHaveBridgeAbove(tile - delta) && IsBridgeAbove(tile - delta)) ||
-	    (MayHaveBridgeAbove(tile + delta) && IsBridgeAbove(tile + delta))) {
+	if (IsBridgeAbove(tile) || IsBridgeAbove(tile - delta) || IsBridgeAbove(tile + delta)) {
 		return_cmd_error(STR_ERROR_MUST_DEMOLISH_BRIDGE_FIRST);
 	}
 
 	if (flags & DC_EXEC) {
-		MakeLock(tile, _current_company, dir, wc_lower, wc_upper);
+		/* Update company infrastructure counts. */
+		Company *c = Company::GetIfValid(_current_company);
+		if (c != NULL) {
+			/* Counts for the water. */
+			if (!IsWaterTile(tile - delta)) c->infrastructure.water++;
+			if (!IsWaterTile(tile + delta)) c->infrastructure.water++;
+			/* Count for the lock itself. */
+			c->infrastructure.water += 3 * LOCK_DEPOT_TILE_FACTOR; // Lock is three tiles.
+			DirtyCompanyInfrastructureWindows(_current_company);
+		}
+
+		MakeLock(tile, _current_company, dir, wc_lower, wc_upper, wc_middle);
 		MarkTileDirtyByTile(tile);
 		MarkTileDirtyByTile(tile - delta);
 		MarkTileDirtyByTile(tile + delta);
@@ -283,9 +336,21 @@ static CommandCost RemoveLock(TileIndex tile, DoCommandFlag flags)
 	if (ret.Failed()) return ret;
 
 	if (flags & DC_EXEC) {
-		DoClearSquare(tile);
+		/* Remove middle part from company infrastructure count. */
+		Company *c = Company::GetIfValid(GetTileOwner(tile));
+		if (c != NULL) {
+			c->infrastructure.water -= 3 * LOCK_DEPOT_TILE_FACTOR; // three parts of the lock.
+			DirtyCompanyInfrastructureWindows(c->index);
+		}
+
+		if (GetWaterClass(tile) == WATER_CLASS_RIVER) {
+			MakeRiver(tile, Random());
+		} else {
+			DoClearSquare(tile);
+		}
 		MakeWaterKeepingClass(tile + delta, GetTileOwner(tile + delta));
 		MakeWaterKeepingClass(tile - delta, GetTileOwner(tile - delta));
+		MarkCanalsAndRiversAroundDirty(tile);
 		MarkCanalsAndRiversAroundDirty(tile - delta);
 		MarkCanalsAndRiversAroundDirty(tile + delta);
 	}
@@ -304,15 +369,19 @@ static CommandCost RemoveLock(TileIndex tile, DoCommandFlag flags)
  */
 CommandCost CmdBuildLock(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	DiagDirection dir = GetInclinedSlopeDirection(GetTileSlope(tile, NULL));
+	DiagDirection dir = GetInclinedSlopeDirection(GetTileSlope(tile));
 	if (dir == INVALID_DIAGDIR) return_cmd_error(STR_ERROR_LAND_SLOPED_IN_WRONG_DIRECTION);
 
-	/* Disallow building of locks on river rapids */
-	if (IsWaterTile(tile)) return_cmd_error(STR_ERROR_SITE_UNSUITABLE);
-
 	return DoBuildLock(tile, dir, flags);
 }
 
+/** Callback to create non-desert around a river tile. */
+bool RiverModifyDesertZone(TileIndex tile, void *)
+{
+	if (GetTropicZone(tile) == TROPICZONE_DESERT) SetTropicZone(tile, TROPICZONE_NORMAL);
+	return false;
+}
+
 /**
  * Build a piece of canal.
  * @param tile end tile of stretch-dragging
@@ -339,7 +408,7 @@ CommandCost CmdBuildCanal(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 	TILE_AREA_LOOP(tile, ta) {
 		CommandCost ret;
 
-		Slope slope = GetTileSlope(tile, NULL);
+		Slope slope = GetTileSlope(tile);
 		if (slope != SLOPE_FLAT && (wc != WATER_CLASS_RIVER || !IsInclinedSlope(slope))) {
 			return_cmd_error(STR_ERROR_FLAT_LAND_REQUIRED);
 		}
@@ -347,14 +416,20 @@ CommandCost CmdBuildCanal(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 		/* can't make water of water! */
 		if (IsTileType(tile, MP_WATER) && (!IsTileOwner(tile, OWNER_WATER) || wc == WATER_CLASS_SEA)) continue;
 
+		bool water = IsWaterTile(tile);
 		ret = DoCommand(tile, 0, 0, flags | DC_FORCE_CLEAR_TILE, CMD_LANDSCAPE_CLEAR);
 		if (ret.Failed()) return ret;
-		cost.AddCost(ret);
+
+		if (!water) cost.AddCost(ret);
 
 		if (flags & DC_EXEC) {
 			switch (wc) {
 				case WATER_CLASS_RIVER:
 					MakeRiver(tile, Random());
+					if (_game_mode == GM_EDITOR) {
+						TileIndex tile2 = tile;
+						CircularTileSearch(&tile2, 5, RiverModifyDesertZone, NULL);
+					}
 					break;
 
 				case WATER_CLASS_SEA:
@@ -366,6 +441,10 @@ CommandCost CmdBuildCanal(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 
 				default:
 					MakeCanal(tile, _current_company, Random());
+					if (Company::IsValidID(_current_company)) {
+						Company::Get(_current_company)->infrastructure.water++;
+						DirtyCompanyInfrastructureWindows(_current_company);
+					}
 					break;
 			}
 			MarkTileDirtyByTile(tile);
@@ -399,12 +478,17 @@ static CommandCost ClearTile_Water(TileIndex tile, DoCommandFlag flags)
 			CommandCost ret = EnsureNoVehicleOnGround(tile);
 			if (ret.Failed()) return ret;
 
-			if (GetTileOwner(tile) != OWNER_WATER && GetTileOwner(tile) != OWNER_NONE) {
+			Owner owner = GetTileOwner(tile);
+			if (owner != OWNER_WATER && owner != OWNER_NONE) {
 				CommandCost ret = CheckTileOwnership(tile);
 				if (ret.Failed()) return ret;
 			}
 
 			if (flags & DC_EXEC) {
+				if (IsCanal(tile) && Company::IsValidID(owner)) {
+					Company::Get(owner)->infrastructure.water--;
+					DirtyCompanyInfrastructureWindows(owner);
+				}
 				DoClearSquare(tile);
 				MarkCanalsAndRiversAroundDirty(tile);
 			}
@@ -413,7 +497,7 @@ static CommandCost ClearTile_Water(TileIndex tile, DoCommandFlag flags)
 		}
 
 		case WATER_TILE_COAST: {
-			Slope slope = GetTileSlope(tile, NULL);
+			Slope slope = GetTileSlope(tile);
 
 			/* Make sure no vehicle is on the tile */
 			CommandCost ret = EnsureNoVehicleOnGround(tile);
@@ -431,16 +515,17 @@ static CommandCost ClearTile_Water(TileIndex tile, DoCommandFlag flags)
 		}
 
 		case WATER_TILE_LOCK: {
-			static const TileIndexDiffC _lock_tomiddle_offs[] = {
-				{ 0,  0}, {0,  0}, { 0, 0}, {0,  0}, // middle
-				{-1,  0}, {0,  1}, { 1, 0}, {0, -1}, // lower
-				{ 1,  0}, {0, -1}, {-1, 0}, {0,  1}, // upper
+			static const TileIndexDiffC _lock_tomiddle_offs[][DIAGDIR_END] = {
+				/*   NE       SE        SW      NW       */
+				{ { 0,  0}, {0,  0}, { 0, 0}, {0,  0} }, // LOCK_PART_MIDDLE
+				{ {-1,  0}, {0,  1}, { 1, 0}, {0, -1} }, // LOCK_PART_LOWER
+				{ { 1,  0}, {0, -1}, {-1, 0}, {0,  1} }, // LOCK_PART_UPPER
 			};
 
 			if (flags & DC_AUTO) return_cmd_error(STR_ERROR_BUILDING_MUST_BE_DEMOLISHED);
 			if (_current_company == OWNER_WATER) return CMD_ERROR;
 			/* move to the middle tile.. */
-			return RemoveLock(tile + ToTileIndexDiff(_lock_tomiddle_offs[GetSection(tile)]), flags);
+			return RemoveLock(tile + ToTileIndexDiff(_lock_tomiddle_offs[GetLockPart(tile)][GetLockDirection(tile)]), flags);
 		}
 
 		case WATER_TILE_DEPOT:
@@ -460,7 +545,7 @@ static CommandCost ClearTile_Water(TileIndex tile, DoCommandFlag flags)
  * @return true iff the tile is water in the view of 'from'.
  *
  */
-static bool IsWateredTile(TileIndex tile, Direction from)
+bool IsWateredTile(TileIndex tile, Direction from)
 {
 	switch (GetTileType(tile)) {
 		case MP_WATER:
@@ -470,7 +555,7 @@ static bool IsWateredTile(TileIndex tile, Direction from)
 				case WATER_TILE_LOCK: return DiagDirToAxis(GetLockDirection(tile)) == DiagDirToAxis(DirToDiagDir(from));
 
 				case WATER_TILE_COAST:
-					switch (GetTileSlope(tile, NULL)) {
+					switch (GetTileSlope(tile)) {
 						case SLOPE_W: return (from == DIR_SE) || (from == DIR_E) || (from == DIR_NE);
 						case SLOPE_S: return (from == DIR_NE) || (from == DIR_N) || (from == DIR_NW);
 						case SLOPE_E: return (from == DIR_NW) || (from == DIR_W) || (from == DIR_SW);
@@ -482,7 +567,7 @@ static bool IsWateredTile(TileIndex tile, Direction from)
 		case MP_RAILWAY:
 			if (GetRailGroundType(tile) == RAIL_GROUND_WATER) {
 				assert(IsPlainRail(tile));
-				switch (GetTileSlope(tile, NULL)) {
+				switch (GetTileSlope(tile)) {
 					case SLOPE_W: return (from == DIR_SE) || (from == DIR_E) || (from == DIR_NE);
 					case SLOPE_S: return (from == DIR_NE) || (from == DIR_N) || (from == DIR_NW);
 					case SLOPE_E: return (from == DIR_NW) || (from == DIR_W) || (from == DIR_SW);
@@ -502,7 +587,7 @@ static bool IsWateredTile(TileIndex tile, Direction from)
 
 				return IsTileOnWater(tile);
 			}
-			return (IsDock(tile) && GetTileSlope(tile, NULL) == SLOPE_FLAT) || IsBuoy(tile);
+			return (IsDock(tile) && IsTileFlat(tile)) || IsBuoy(tile);
 
 		case MP_INDUSTRY: {
 			/* Do not draw waterborders inside of industries.
@@ -518,6 +603,8 @@ static bool IsWateredTile(TileIndex tile, Direction from)
 
 		case MP_TUNNELBRIDGE: return GetTunnelBridgeTransportType(tile) == TRANSPORT_WATER && ReverseDiagDir(GetTunnelBridgeDirection(tile)) == DirToDiagDir(from);
 
+		case MP_VOID: return true; // consider map border as water, esp. for rivers
+
 		default:          return false;
 	}
 }
@@ -616,35 +703,29 @@ static void DrawCanalWater(TileIndex tile)
 	DrawWaterEdges(true, 0, tile);
 }
 
-struct LocksDrawTileStruct {
-	int8 delta_x, delta_y, delta_z;
-	byte width, height, depth;
-	SpriteID image;
-};
-
 #include "table/water_land.h"
 
 /**
  * Draw a build sprite sequence for water tiles.
  * If buildings are invisible, nothing will be drawn.
  * @param ti      Tile info.
- * @param wdts    Sprite sequence to draw.
+ * @param dtss     Sprite sequence to draw.
  * @param base    Base sprite.
  * @param offset  Additional sprite offset.
  * @param palette Palette to use.
  */
-static void DrawWaterTileStruct(const TileInfo *ti, const WaterDrawTileStruct *wdts, SpriteID base, uint offset, PaletteID palette, CanalFeature feature)
+static void DrawWaterTileStruct(const TileInfo *ti, const DrawTileSeqStruct *dtss, SpriteID base, uint offset, PaletteID palette, CanalFeature feature)
 {
 	/* Don't draw if buildings are invisible. */
 	if (IsInvisibilitySet(TO_BUILDINGS)) return;
 
-	for (; wdts->delta_x != 0x80; wdts++) {
-		uint tile_offs = offset + wdts->image;
+	for (; !dtss->IsTerminator(); dtss++) {
+		uint tile_offs = offset + dtss->image.sprite;
 		if (feature < CF_END) tile_offs = GetCanalSpriteOffset(feature, ti->tile, tile_offs);
 		AddSortableSpriteToDraw(base + tile_offs, palette,
-			ti->x + wdts->delta_x, ti->y + wdts->delta_y,
-			wdts->size_x, wdts->size_y,
-			wdts->size_z, ti->z + wdts->delta_z,
+			ti->x + dtss->delta_x, ti->y + dtss->delta_y,
+			dtss->size_x, dtss->size_y,
+			dtss->size_z, ti->z + dtss->delta_z,
 			IsTransparencySet(TO_BUILDINGS));
 	}
 }
@@ -652,11 +733,11 @@ static void DrawWaterTileStruct(const TileInfo *ti, const WaterDrawTileStruct *w
 /** Draw a lock tile. */
 static void DrawWaterLock(const TileInfo *ti)
 {
-	int section = GetSection(ti->tile);
-	const WaterDrawTileStruct *wdts = _lock_display_seq[section];
+	int part = GetLockPart(ti->tile);
+	const DrawTileSprites &dts = _lock_display_data[part][GetLockDirection(ti->tile)];
 
 	/* Draw ground sprite. */
-	SpriteID image = wdts++->image;
+	SpriteID image = dts.ground.sprite;
 
 	SpriteID water_base = GetCanalSprite(CF_WATERSLOPE, ti->tile);
 	if (water_base == 0) {
@@ -681,19 +762,18 @@ static void DrawWaterLock(const TileInfo *ti)
 	if (base == 0) {
 		/* If no custom graphics, use defaults. */
 		base = SPR_LOCK_BASE;
-		uint8 z_threshold = section >= 8 ? 8 : 0;
+		uint8 z_threshold = part == LOCK_PART_UPPER ? 8 : 0;
 		zoffs = ti->z > z_threshold ? 24 : 0;
 	}
 
-	DrawWaterTileStruct(ti, wdts, base, zoffs, PAL_NONE, CF_LOCKS);
+	DrawWaterTileStruct(ti, dts.seq, base, zoffs, PAL_NONE, CF_LOCKS);
 }
 
 /** Draw a ship depot tile. */
 static void DrawWaterDepot(const TileInfo *ti)
 {
 	DrawWaterClassGround(ti);
-	/* Skip first entry in _shipdepot_display_seq as this is the ground sprite. */
-	DrawWaterTileStruct(ti, _shipdepot_display_seq[GetSection(ti->tile)] + 1, 0, 0, COMPANY_SPRITE_COLOUR(GetTileOwner(ti->tile)), CF_END);
+	DrawWaterTileStruct(ti, _shipdepot_display_data[GetShipDepotAxis(ti->tile)][GetShipDepotPart(ti->tile)].seq, 0, 0, COMPANY_SPRITE_COLOUR(GetTileOwner(ti->tile)), CF_END);
 }
 
 static void DrawRiverWater(const TileInfo *ti)
@@ -785,25 +865,21 @@ static void DrawTile_Water(TileInfo *ti)
 	}
 }
 
-void DrawShipDepotSprite(int x, int y, int image)
+void DrawShipDepotSprite(int x, int y, Axis axis, DepotPart part)
 {
-	const WaterDrawTileStruct *wdts = _shipdepot_display_seq[image];
+	const DrawTileSprites &dts = _shipdepot_display_data[axis][part];
 
-	DrawSprite(wdts++->image, PAL_NONE, x, y);
-
-	for (; wdts->delta_x != 0x80; wdts++) {
-		Point pt = RemapCoords(wdts->delta_x, wdts->delta_y, wdts->delta_z);
-		DrawSprite(wdts->image, COMPANY_SPRITE_COLOUR(_local_company), x + pt.x, y + pt.y);
-	}
+	DrawSprite(dts.ground.sprite, dts.ground.pal, x, y);
+	DrawOrigTileSeqInGUI(x, y, &dts, COMPANY_SPRITE_COLOUR(_local_company));
 }
 
 
-static uint GetSlopeZ_Water(TileIndex tile, uint x, uint y)
+static int GetSlopePixelZ_Water(TileIndex tile, uint x, uint y)
 {
-	uint z;
-	Slope tileh = GetTileSlope(tile, &z);
+	int z;
+	Slope tileh = GetTilePixelSlope(tile, &z);
 
-	return z + GetPartialZ(x & 0xF, y & 0xF, tileh);
+	return z + GetPartialPixelZ(x & 0xF, y & 0xF, tileh);
 }
 
 static Foundation GetFoundation_Water(TileIndex tile, Slope tileh)
@@ -843,11 +919,12 @@ static void FloodVehicle(Vehicle *v)
 {
 	uint pass = v->Crash(true);
 
-	AI::NewEvent(v->owner, new AIEventVehicleCrashed(v->index, v->tile, AIEventVehicleCrashed::CRASH_FLOODED));
+	AI::NewEvent(v->owner, new ScriptEventVehicleCrashed(v->index, v->tile, ScriptEventVehicleCrashed::CRASH_FLOODED));
+	Game::NewEvent(new ScriptEventVehicleCrashed(v->index, v->tile, ScriptEventVehicleCrashed::CRASH_FLOODED));
 	SetDParam(0, pass);
-	AddVehicleNewsItem(STR_NEWS_DISASTER_FLOOD_VEHICLE, NS_ACCIDENT, v->index);
+	AddVehicleNewsItem(STR_NEWS_DISASTER_FLOOD_VEHICLE, NT_ACCIDENT, v->index);
 	CreateEffectVehicleRel(v, 4, 4, 8, EV_EXPLOSION_LARGE);
-	SndPlayVehicleFx(SND_12_EXPLOSION, v);
+	if (_settings_client.sound.disaster) SndPlayVehicleFx(SND_12_EXPLOSION, v);
 }
 
 /**
@@ -879,7 +956,7 @@ static Vehicle *FloodVehicleProc(Vehicle *v, void *data)
 
 		case VEH_TRAIN:
 		case VEH_ROAD: {
-			byte z = *(byte*)data;
+			int z = *(int*)data;
 			if (v->z_pos > z) break;
 			FloodVehicle(v->First());
 			break;
@@ -896,7 +973,7 @@ static Vehicle *FloodVehicleProc(Vehicle *v, void *data)
  */
 static void FloodVehicles(TileIndex tile)
 {
-	byte z = 0;
+	int z = 0;
 
 	if (IsAirportTile(tile)) {
 		const Station *st = Station::GetByTile(tile);
@@ -914,7 +991,7 @@ static void FloodVehicles(TileIndex tile)
 	}
 
 	TileIndex end = GetOtherBridgeEnd(tile);
-	z = GetBridgeHeight(tile);
+	z = GetBridgePixelHeight(tile);
 
 	FindVehicleOnPos(tile, &z, &FloodVehicleProc);
 	FindVehicleOnPos(end, &z, &FloodVehicleProc);
@@ -935,7 +1012,7 @@ FloodingBehaviour GetFloodingBehaviour(TileIndex tile)
 	switch (GetTileType(tile)) {
 		case MP_WATER:
 			if (IsCoast(tile)) {
-				Slope tileh = GetTileSlope(tile, NULL);
+				Slope tileh = GetTileSlope(tile);
 				return (IsSlopeWithOneCornerRaised(tileh) ? FLOOD_ACTIVE : FLOOD_DRYUP);
 			}
 			/* FALL THROUGH */
@@ -946,7 +1023,7 @@ FloodingBehaviour GetFloodingBehaviour(TileIndex tile)
 
 		case MP_RAILWAY:
 			if (GetRailGroundType(tile) == RAIL_GROUND_WATER) {
-				return (IsSlopeWithOneCornerRaised(GetTileSlope(tile, NULL)) ? FLOOD_ACTIVE : FLOOD_DRYUP);
+				return (IsSlopeWithOneCornerRaised(GetTileSlope(tile)) ? FLOOD_ACTIVE : FLOOD_DRYUP);
 			}
 			return FLOOD_NONE;
 
@@ -969,7 +1046,7 @@ void DoFloodTile(TileIndex target)
 
 	Backup<CompanyByte> cur_company(_current_company, OWNER_WATER, FILE_LINE);
 
-	Slope tileh = GetTileSlope(target, NULL);
+	Slope tileh = GetTileSlope(target);
 	if (tileh != SLOPE_FLAT) {
 		/* make coast.. */
 		switch (GetTileType(target)) {
@@ -1075,15 +1152,20 @@ static void DoDryUp(TileIndex tile)
  */
 void TileLoop_Water(TileIndex tile)
 {
+	if (IsTileType(tile, MP_WATER)) AmbientSoundEffect(tile);
+
 	switch (GetFloodingBehaviour(tile)) {
 		case FLOOD_ACTIVE:
 			for (Direction dir = DIR_BEGIN; dir < DIR_END; dir++) {
-				TileIndex dest = AddTileIndexDiffCWrap(tile, TileIndexDiffCByDir(dir));
-				if (dest == INVALID_TILE) continue;
+				TileIndex dest = tile + TileOffsByDir(dir);
+				if (!IsValidTile(dest)) continue;
 				/* do not try to flood water tiles - increases performance a lot */
 				if (IsTileType(dest, MP_WATER)) continue;
 
-				uint z_dest;
+				/* TREE_GROUND_SHORE is the sign of a previous flood. */
+				if (IsTileType(dest, MP_TREES) && GetTreeGround(dest) == TREE_GROUND_SHORE) continue;
+
+				int z_dest;
 				Slope slope_dest = GetFoundationSlope(dest, &z_dest) & ~SLOPE_HALFTILE_MASK & ~SLOPE_STEEP;
 				if (z_dest > 0) continue;
 
@@ -1094,11 +1176,11 @@ void TileLoop_Water(TileIndex tile)
 			break;
 
 		case FLOOD_DRYUP: {
-			Slope slope_here = GetFoundationSlope(tile, NULL) & ~SLOPE_HALFTILE_MASK & ~SLOPE_STEEP;
+			Slope slope_here = GetFoundationSlope(tile) & ~SLOPE_HALFTILE_MASK & ~SLOPE_STEEP;
 			uint dir;
 			FOR_EACH_SET_BIT(dir, _flood_from_dirs[slope_here]) {
-				TileIndex dest = AddTileIndexDiffCWrap(tile, TileIndexDiffCByDir((Direction)dir));
-				if (dest == INVALID_TILE) continue;
+				TileIndex dest = tile + TileOffsByDir((Direction)dir);
+				if (!IsValidTile(dest)) continue;
 
 				FloodingBehaviour dest_behaviour = GetFloodingBehaviour(dest);
 				if ((dest_behaviour == FLOOD_ACTIVE) || (dest_behaviour == FLOOD_PASSIVE)) return;
@@ -1113,7 +1195,7 @@ void TileLoop_Water(TileIndex tile)
 
 void ConvertGroundTilesIntoWaterTiles()
 {
-	uint z;
+	int z;
 
 	for (TileIndex tile = 0; tile < MapSize(); ++tile) {
 		Slope slope = GetTileSlope(tile, &z);
@@ -1137,7 +1219,7 @@ void ConvertGroundTilesIntoWaterTiles()
 					uint dir;
 					FOR_EACH_SET_BIT(dir, _flood_from_dirs[slope & ~SLOPE_STEEP]) {
 						TileIndex dest = TILE_ADD(tile, TileOffsByDir((Direction)dir));
-						Slope slope_dest = GetTileSlope(dest, NULL) & ~SLOPE_STEEP;
+						Slope slope_dest = GetTileSlope(dest) & ~SLOPE_STEEP;
 						if (slope_dest == SLOPE_FLAT || IsSlopeWithOneCornerRaised(slope_dest)) {
 							MakeShore(tile);
 							break;
@@ -1158,8 +1240,8 @@ static TrackStatus GetTileTrackStatus_Water(TileIndex tile, TransportType mode,
 	if (mode != TRANSPORT_WATER) return 0;
 
 	switch (GetWaterTileType(tile)) {
-		case WATER_TILE_CLEAR: ts = (GetTileSlope(tile, NULL) == SLOPE_FLAT) ? TRACK_BIT_ALL : TRACK_BIT_NONE; break;
-		case WATER_TILE_COAST: ts = (TrackBits)coast_tracks[GetTileSlope(tile, NULL) & 0xF]; break;
+		case WATER_TILE_CLEAR: ts = IsTileFlat(tile) ? TRACK_BIT_ALL : TRACK_BIT_NONE; break;
+		case WATER_TILE_COAST: ts = (TrackBits)coast_tracks[GetTileSlope(tile) & 0xF]; break;
 		case WATER_TILE_LOCK:  ts = DiagDirToDiagTrackBits(GetLockDirection(tile)); break;
 		case WATER_TILE_DEPOT: ts = AxisToTrackBits(GetShipDepotAxis(tile)); break;
 		default: return 0;
@@ -1188,7 +1270,23 @@ static void ChangeTileOwner_Water(TileIndex tile, Owner old_owner, Owner new_own
 {
 	if (!IsTileOwner(tile, old_owner)) return;
 
+	bool is_lock_middle = IsLock(tile) && GetLockPart(tile) == LOCK_PART_MIDDLE;
+
+	/* No need to dirty company windows here, we'll redraw the whole screen anyway. */
+	if (is_lock_middle) Company::Get(old_owner)->infrastructure.water -= 3 * LOCK_DEPOT_TILE_FACTOR; // Lock has three parts.
 	if (new_owner != INVALID_OWNER) {
+		if (is_lock_middle) Company::Get(new_owner)->infrastructure.water += 3 * LOCK_DEPOT_TILE_FACTOR; // Lock has three parts.
+		/* Only subtract from the old owner here if the new owner is valid,
+		 * otherwise we clear ship depots and canal water below. */
+		if (GetWaterClass(tile) == WATER_CLASS_CANAL && !is_lock_middle) {
+			Company::Get(old_owner)->infrastructure.water--;
+			Company::Get(new_owner)->infrastructure.water++;
+		}
+		if (IsShipDepot(tile)) {
+			Company::Get(old_owner)->infrastructure.water -= LOCK_DEPOT_TILE_FACTOR;
+			Company::Get(new_owner)->infrastructure.water += LOCK_DEPOT_TILE_FACTOR;
+		}
+
 		SetTileOwner(tile, new_owner);
 		return;
 	}
@@ -1198,7 +1296,10 @@ static void ChangeTileOwner_Water(TileIndex tile, Owner old_owner, Owner new_own
 
 	/* Set owner of canals and locks ... and also canal under dock there was before.
 	 * Check if the new owner after removing depot isn't OWNER_WATER. */
-	if (IsTileOwner(tile, old_owner)) SetTileOwner(tile, OWNER_NONE);
+	if (IsTileOwner(tile, old_owner)) {
+		if (GetWaterClass(tile) == WATER_CLASS_CANAL && !is_lock_middle) Company::Get(old_owner)->infrastructure.water--;
+		SetTileOwner(tile, OWNER_NONE);
+	}
 }
 
 static VehicleEnterTileStatus VehicleEnter_Water(Vehicle *v, TileIndex tile, int x, int y)
@@ -1206,7 +1307,7 @@ static VehicleEnterTileStatus VehicleEnter_Water(Vehicle *v, TileIndex tile, int
 	return VETSB_CONTINUE;
 }
 
-static CommandCost TerraformTile_Water(TileIndex tile, DoCommandFlag flags, uint z_new, Slope tileh_new)
+static CommandCost TerraformTile_Water(TileIndex tile, DoCommandFlag flags, int z_new, Slope tileh_new)
 {
 	/* Canals can't be terraformed */
 	if (IsWaterTile(tile) && IsCanal(tile)) return_cmd_error(STR_ERROR_MUST_DEMOLISH_CANAL_FIRST);
@@ -1217,15 +1318,15 @@ static CommandCost TerraformTile_Water(TileIndex tile, DoCommandFlag flags, uint
 
 extern const TileTypeProcs _tile_type_water_procs = {
 	DrawTile_Water,           // draw_tile_proc
-	GetSlopeZ_Water,          // get_slope_z_proc
+	GetSlopePixelZ_Water,     // get_slope_z_proc
 	ClearTile_Water,          // clear_tile_proc
 	NULL,                     // add_accepted_cargo_proc
 	GetTileDesc_Water,        // get_tile_desc_proc
 	GetTileTrackStatus_Water, // get_tile_track_status_proc
 	ClickTile_Water,          // click_tile_proc
 	NULL,                     // animate_tile_proc
-	TileLoop_Water,           // tile_loop_clear
-	ChangeTileOwner_Water,    // change_tile_owner_clear
+	TileLoop_Water,           // tile_loop_proc
+	ChangeTileOwner_Water,    // change_tile_owner_proc
 	NULL,                     // add_produced_cargo_proc
 	VehicleEnter_Water,       // vehicle_enter_tile_proc
 	GetFoundation_Water,      // get_foundation_proc
