@@ -1,4 +1,4 @@
-/* $Id: waypoint_gui.cpp 22252 2011-03-14 19:03:17Z rubidium $ */
+/* $Id: waypoint_gui.cpp 27030 2014-10-21 19:16:47Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -23,16 +23,11 @@
 #include "window_func.h"
 #include "waypoint_base.h"
 
+#include "widgets/waypoint_widget.h"
+
 #include "table/strings.h"
 
-/** Widget definitions for the waypoint window. */
-enum WaypointWindowWidgets {
-	WAYPVW_CAPTION,
-	WAYPVW_VIEWPORT,
-	WAYPVW_CENTERVIEW,
-	WAYPVW_RENAME,
-	WAYPVW_SHOW_VEHICLES,
-};
+#include "safeguards.h"
 
 /** GUI for accessing waypoints and buoys. */
 struct WaypointWindow : Window {
@@ -54,48 +49,47 @@ private:
 	}
 
 public:
-	WaypointWindow(const WindowDesc *desc, WindowNumber window_number) : Window()
+	/**
+	 * Construct the window.
+	 * @param desc The description of the window.
+	 * @param window_number The window number, in this case the waypoint's ID.
+	 */
+	WaypointWindow(WindowDesc *desc, WindowNumber window_number) : Window(desc)
 	{
 		this->wp = Waypoint::Get(window_number);
 		this->vt = (wp->string_id == STR_SV_STNAME_WAYPOINT) ? VEH_TRAIN : VEH_SHIP;
 
-		this->CreateNestedTree(desc);
+		this->CreateNestedTree();
 		if (this->vt == VEH_TRAIN) {
-			this->GetWidget<NWidgetCore>(WAYPVW_SHOW_VEHICLES)->SetDataTip(STR_TRAIN, STR_STATION_VIEW_SCHEDULED_TRAINS_TOOLTIP);
-			this->GetWidget<NWidgetCore>(WAYPVW_CENTERVIEW)->tool_tip = STR_WAYPOINT_VIEW_CENTER_TOOLTIP;
-			this->GetWidget<NWidgetCore>(WAYPVW_RENAME)->tool_tip = STR_WAYPOINT_VIEW_CHANGE_WAYPOINT_NAME;
+			this->GetWidget<NWidgetCore>(WID_W_SHOW_VEHICLES)->SetDataTip(STR_TRAIN, STR_STATION_VIEW_SCHEDULED_TRAINS_TOOLTIP);
+			this->GetWidget<NWidgetCore>(WID_W_CENTER_VIEW)->tool_tip = STR_WAYPOINT_VIEW_CENTER_TOOLTIP;
+			this->GetWidget<NWidgetCore>(WID_W_RENAME)->tool_tip = STR_WAYPOINT_VIEW_CHANGE_WAYPOINT_NAME;
 		}
-		this->FinishInitNested(desc, window_number);
+		this->FinishInitNested(window_number);
 
-		if (this->wp->owner != OWNER_NONE) this->owner = this->wp->owner;
-		this->flags4 |= WF_DISABLE_VP_SCROLL;
+		this->owner = this->wp->owner;
+		this->flags |= WF_DISABLE_VP_SCROLL;
 
-		NWidgetViewport *nvp = this->GetWidget<NWidgetViewport>(WAYPVW_VIEWPORT);
-		nvp->InitializeViewport(this, this->GetCenterTile(), ZOOM_LVL_MIN);
+		NWidgetViewport *nvp = this->GetWidget<NWidgetViewport>(WID_W_VIEWPORT);
+		nvp->InitializeViewport(this, this->GetCenterTile(), ZOOM_LVL_VIEWPORT);
 
 		this->OnInvalidateData(0);
 	}
 
 	~WaypointWindow()
 	{
-		Owner owner = this->owner;
-
-		/* Buoys have no owner and can be used by everyone. Show only 'our' vehicles */
-		if (!Company::IsValidID(owner)) owner = _local_company;
-
-		/* Well, spectators otoh */
-		if (Company::IsValidID(owner)) DeleteWindowById(GetWindowClassForVehicleType(this->vt), VehicleListIdentifier(VL_STATION_LIST, this->vt, owner, this->window_number).Pack(), false);
+		DeleteWindowById(GetWindowClassForVehicleType(this->vt), VehicleListIdentifier(VL_STATION_LIST, this->vt, this->owner, this->window_number).Pack(), false);
 	}
 
 	virtual void SetStringParameters(int widget) const
 	{
-		if (widget == WAYPVW_CAPTION) SetDParam(0, this->wp->index);
+		if (widget == WID_W_CAPTION) SetDParam(0, this->wp->index);
 	}
 
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		switch (widget) {
-			case WAYPVW_CENTERVIEW: // scroll to location
+			case WID_W_CENTER_VIEW: // scroll to location
 				if (_ctrl_pressed) {
 					ShowExtraViewPortWindow(this->GetCenterTile());
 				} else {
@@ -103,13 +97,13 @@ public:
 				}
 				break;
 
-			case WAYPVW_RENAME: // rename
+			case WID_W_RENAME: // rename
 				SetDParam(0, this->wp->index);
-				ShowQueryString(STR_WAYPOINT_NAME, STR_EDIT_WAYPOINT_NAME, MAX_LENGTH_STATION_NAME_CHARS, MAX_LENGTH_STATION_NAME_PIXELS, this, CS_ALPHANUMERAL, QSF_ENABLE_DEFAULT | QSF_LEN_IN_CHARS);
+				ShowQueryString(STR_WAYPOINT_NAME, STR_EDIT_WAYPOINT_NAME, MAX_LENGTH_STATION_NAME_CHARS, this, CS_ALPHANUMERAL, QSF_ENABLE_DEFAULT | QSF_LEN_IN_CHARS);
 				break;
 
-			case WAYPVW_SHOW_VEHICLES: // show list of vehicles having this waypoint in their orders
-				ShowVehicleListWindow(this->owner, this->vt, this->wp->index);
+			case WID_W_SHOW_VEHICLES: // show list of vehicles having this waypoint in their orders
+				ShowVehicleListWindow(this->wp->owner, this->vt, this->wp->index);
 				break;
 		}
 	}
@@ -123,9 +117,9 @@ public:
 	{
 		if (!gui_scope) return;
 		/* You can only change your own waypoints */
-		this->SetWidgetDisabledState(WAYPVW_RENAME, !this->wp->IsInUse() || (this->wp->owner != _local_company && this->wp->owner != OWNER_NONE));
+		this->SetWidgetDisabledState(WID_W_RENAME, !this->wp->IsInUse() || (this->wp->owner != _local_company && this->wp->owner != OWNER_NONE));
 		/* Disable the widget for waypoints with no use */
-		this->SetWidgetDisabledState(WAYPVW_SHOW_VEHICLES, !this->wp->IsInUse());
+		this->SetWidgetDisabledState(WID_W_SHOW_VEHICLES, !this->wp->IsInUse());
 
 		ScrollWindowToTile(this->GetCenterTile(), this, true);
 	}
@@ -133,7 +127,7 @@ public:
 	virtual void OnResize()
 	{
 		if (this->viewport != NULL) {
-			NWidgetViewport *nvp = this->GetWidget<NWidgetViewport>(WAYPVW_VIEWPORT);
+			NWidgetViewport *nvp = this->GetWidget<NWidgetViewport>(WID_W_VIEWPORT);
 			nvp->UpdateViewportCoordinates(this);
 			this->wp->UpdateVirtCoord();
 
@@ -150,33 +144,40 @@ public:
 
 };
 
+/** The widgets of the waypoint view. */
 static const NWidgetPart _nested_waypoint_view_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
-		NWidget(WWT_CAPTION, COLOUR_GREY, WAYPVW_CAPTION), SetDataTip(STR_WAYPOINT_VIEW_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
+		NWidget(WWT_CAPTION, COLOUR_GREY, WID_W_CAPTION), SetDataTip(STR_WAYPOINT_VIEW_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
+		NWidget(WWT_DEFSIZEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(WWT_PANEL, COLOUR_GREY),
 		NWidget(WWT_INSET, COLOUR_GREY), SetPadding(2, 2, 2, 2),
-			NWidget(NWID_VIEWPORT, COLOUR_GREY, WAYPVW_VIEWPORT), SetMinimalSize(256, 88), SetPadding(1, 1, 1, 1), SetResize(1, 1),
+			NWidget(NWID_VIEWPORT, COLOUR_GREY, WID_W_VIEWPORT), SetMinimalSize(256, 88), SetPadding(1, 1, 1, 1), SetResize(1, 1),
 		EndContainer(),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WAYPVW_CENTERVIEW), SetMinimalSize(100, 12), SetResize(1, 0), SetFill(1, 0), SetDataTip(STR_BUTTON_LOCATION, STR_BUOY_VIEW_CENTER_TOOLTIP),
-		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WAYPVW_RENAME), SetMinimalSize(100, 12), SetResize(1, 0), SetFill(1, 0), SetDataTip(STR_BUTTON_RENAME, STR_BUOY_VIEW_CHANGE_BUOY_NAME),
-		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WAYPVW_SHOW_VEHICLES), SetMinimalSize(15, 12), SetDataTip(STR_SHIP, STR_STATION_VIEW_SCHEDULED_SHIPS_TOOLTIP),
+		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_W_CENTER_VIEW), SetMinimalSize(100, 12), SetResize(1, 0), SetFill(1, 0), SetDataTip(STR_BUTTON_LOCATION, STR_BUOY_VIEW_CENTER_TOOLTIP),
+		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_W_RENAME), SetMinimalSize(100, 12), SetResize(1, 0), SetFill(1, 0), SetDataTip(STR_BUTTON_RENAME, STR_BUOY_VIEW_CHANGE_BUOY_NAME),
+		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_W_SHOW_VEHICLES), SetMinimalSize(15, 12), SetDataTip(STR_SHIP, STR_STATION_VIEW_SCHEDULED_SHIPS_TOOLTIP),
 		NWidget(WWT_RESIZEBOX, COLOUR_GREY),
 	EndContainer(),
 };
 
-static const WindowDesc _waypoint_view_desc(
-	WDP_AUTO, 260, 118,
+/** The description of the waypoint view. */
+static WindowDesc _waypoint_view_desc(
+	WDP_AUTO, "view_waypoint", 260, 118,
 	WC_WAYPOINT_VIEW, WC_NONE,
-	WDF_UNCLICK_BUTTONS,
+	0,
 	_nested_waypoint_view_widgets, lengthof(_nested_waypoint_view_widgets)
 );
 
+/**
+ * Show the window for the given waypoint.
+ * @param wp The waypoint to show the window for.
+ */
 void ShowWaypointWindow(const Waypoint *wp)
 {
 	AllocateWindowDescFront<WaypointWindow>(&_waypoint_view_desc, wp->index);
